@@ -32,6 +32,7 @@ use Koha::Acquisition::Baskets;
 use Koha::Acquisition::Booksellers;
 use Koha::Acquisition::Invoices;
 use Koha::Acquisition::Orders;
+use Koha::AdditionalFieldValue;
 use Koha::Biblios;
 use Koha::Exceptions;
 use Koha::Items;
@@ -71,8 +72,6 @@ BEGIN {
       TransferOrder
       ModItemOrder
 
-      GetParcels
-
       GetInvoices
       GetInvoice
       GetInvoiceDetails
@@ -1391,6 +1390,9 @@ sub ModReceiveOrder {
     $datereceived = $datereceived ? dt_from_string( $datereceived ) : dt_from_string;
     $datereceived = $datereceived->ymd;
 
+    $order->{invoice_unitprice} ||= $order->{unitprice};
+    $order->{invoice_currency}  ||= Koha::Acquisition::Currencies->get_active->currency;
+
     my $suggestionid = GetSuggestionFromBiblionumber( $biblionumber );
     if ($suggestionid) {
         ModSuggestion( {suggestionid=>$suggestionid,
@@ -1423,23 +1425,29 @@ sub ModReceiveOrder {
             $order->{ordernumber}
         );
 
-        if ( not $order->{subscriptionid} && defined $order->{order_internalnote} ) {
-            $dbh->do(
-                q|UPDATE aqorders
-                SET order_internalnote = ?
-                WHERE ordernumber = ?|, {},
-                $order->{order_internalnote}, $order->{ordernumber}
-            );
-        }
-
         # Recalculate tax_value
-        $dbh->do(q|
+        $query = q|
             UPDATE aqorders
             SET
                 tax_value_on_ordering = quantity * | . get_rounding_sql(q|ecost_tax_excluded|) . q| * tax_rate_on_ordering,
                 tax_value_on_receiving = quantity * | . get_rounding_sql(q|unitprice_tax_excluded|) . q| * tax_rate_on_receiving
+        |;
+
+        my @params;
+        if ( not $order->{subscriptionid} && defined $order->{order_internalnote} )
+        {
+            $query .= q|, order_internalnote = ?|;
+            push @params, $order->{order_internalnote};
+        }
+
+        $query .= q|, invoice_unitprice = ?, invoice_currency = ?|;
+        push @params, $order->{invoice_unitprice}, $order->{invoice_currency};
+
+        $query .= q|
             WHERE ordernumber = ?
-        |, undef, $order->{ordernumber});
+        |;
+
+        $dbh->do($query, undef, @params, $order->{ordernumber});
 
         delete $order->{ordernumber};
         $order->{budget_id} = ( $budget_id || $order->{budget_id} );
@@ -1491,6 +1499,10 @@ sub ModReceiveOrder {
             , order_internalnote = ?
         | if defined $order->{order_internalnote};
 
+        $query .= q|
+            , invoice_unitprice = ?, invoice_currency = ?
+        |;
+
         $query .= q| where biblionumber=? and ordernumber=?|;
 
         my $sth = $dbh->prepare( $query );
@@ -1516,6 +1528,8 @@ sub ModReceiveOrder {
             push @params, $order->{order_internalnote};
         }
 
+        push @params, $order->{invoice_unitprice}, $order->{invoice_currency};
+
         push @params, ( $biblionumber, $order->{ordernumber} );
 
         $sth->execute( @params );
@@ -1873,7 +1887,7 @@ sub TransferOrder {
     my $order = Koha::Acquisition::Orders->find( $ordernumber ) or return;
     return if $order->datereceived;
 
-    $order = $order->unblessed;
+    my $orderhash = $order->unblessed;
 
     my $basket = GetBasket($basketno);
     return unless $basket;
@@ -1889,11 +1903,12 @@ sub TransferOrder {
     $sth = $dbh->prepare($query);
     $rv = $sth->execute('cancelled', $ordernumber);
 
-    delete $order->{'ordernumber'};
-    delete $order->{parent_ordernumber};
-    $order->{'basketno'} = $basketno;
+    delete $orderhash->{ordernumber};
+    delete $orderhash->{parent_ordernumber};
+    $orderhash->{basketno} = $basketno;
 
-    my $newordernumber = Koha::Acquisition::Order->new($order)->store->ordernumber;
+    my $neworder = Koha::Acquisition::Order->new($orderhash);
+    my $newordernumber = $neworder->store->ordernumber;
 
     $query = q{
         UPDATE aqorders_items
@@ -1910,6 +1925,15 @@ sub TransferOrder {
     $sth = $dbh->prepare($query);
     $sth->execute($ordernumber, $newordernumber);
 
+    # Copy additional fields values
+    foreach my $afv ($order->additional_field_values->as_list) {
+        Koha::AdditionalFieldValue->new({
+            field_id => $afv->field_id,
+            record_id => $newordernumber,
+            value => $afv->value,
+        })->store;
+    }
+
     return $newordernumber;
 }
 
@@ -1947,97 +1971,6 @@ sub get_rounded_price {
     return $price;
 }
 
-
-=head2 FUNCTIONS ABOUT PARCELS
-
-=head3 GetParcels
-
-  $results = &GetParcels($bookseller, $order, $code, $datefrom, $dateto);
-
-get a lists of parcels.
-
-* Input arg :
-
-=over
-
-=item $bookseller
-is the bookseller this function has to get parcels.
-
-=item $order
-To know on what criteria the results list has to be ordered.
-
-=item $code
-is the booksellerinvoicenumber.
-
-=item $datefrom & $dateto
-to know on what date this function has to filter its search.
-
-=back
-
-* return:
-a pointer on a hash list containing parcel informations as such :
-
-=over
-
-=item Creation date
-
-=item Last operation
-
-=item Number of biblio
-
-=item Number of items
-
-=back
-
-=cut
-
-sub GetParcels {
-    my ($bookseller,$order, $code, $datefrom, $dateto) = @_;
-    my $dbh    = C4::Context->dbh;
-    my @query_params = ();
-    my $strsth ="
-        SELECT  aqinvoices.invoicenumber,
-                datereceived,purchaseordernumber,
-                count(DISTINCT biblionumber) AS biblio,
-                sum(quantity) AS itemsexpected,
-                sum(quantityreceived) AS itemsreceived
-        FROM   aqorders LEFT JOIN aqbasket ON aqbasket.basketno = aqorders.basketno
-        LEFT JOIN aqinvoices ON aqorders.invoiceid = aqinvoices.invoiceid
-        WHERE aqbasket.booksellerid = ? and datereceived IS NOT NULL
-    ";
-    push @query_params, $bookseller;
-
-    if ( defined $code ) {
-        $strsth .= ' and aqinvoices.invoicenumber like ? ';
-        # add a % to the end of the code to allow stemming.
-        push @query_params, "$code%";
-    }
-
-    if ( defined $datefrom ) {
-        $strsth .= ' and datereceived >= ? ';
-        push @query_params, $datefrom;
-    }
-
-    if ( defined $dateto ) {
-        $strsth .=  'and datereceived <= ? ';
-        push @query_params, $dateto;
-    }
-
-    $strsth .= "group by aqinvoices.invoicenumber,datereceived ";
-
-    # can't use a placeholder to place this column name.
-    # but, we could probably be checking to make sure it is a column that will be fetched.
-    $strsth .= "order by $order " if ($order);
-
-    my $sth = $dbh->prepare($strsth);
-
-    $sth->execute( @query_params );
-    my $results = $sth->fetchall_arrayref({});
-    return @{$results};
-}
-
-#------------------------------------------------------------#
-
 =head3 GetHistory
 
   \@order_loop = GetHistory( %params );
@@ -2259,7 +2192,7 @@ sub GetHistory {
         push @query_params, $to_placed_on;
     }
 
-    if ( defined $orderstatus and $orderstatus ne '') {
+    if ( defined $orderstatus and $orderstatus ne '' and $orderstatus ne 'any') {
         $query .= " AND aqorders.orderstatus = ? ";
         push @query_params, "$orderstatus";
     }
