@@ -342,7 +342,7 @@ the value cannot be properly decoded as YAML.
 sub yaml_preference {
     my ( $self, $preference ) = @_;
 
-    my $yaml = eval { YAML::XS::Load( Encode::encode_utf8( $self->preference( $preference ) ) ); };
+    my $yaml = eval { YAML::XS::Load( Encode::encode_utf8( $self->preference( $preference ) // '' ) ); };
     if ($@) {
         warn "Unable to parse $preference syspref : $@";
         return;
@@ -488,6 +488,7 @@ was no syspref of the name.
 sub delete_preference {
     my ( $self, $var ) = @_;
 
+    $var = lc $var;
     if ( Koha::Config::SysPrefs->find( $var )->delete ) {
         if ( $use_syspref_cache ) {
             my $syspref_cache = Koha::Caches->get_instance('syspref');
