@@ -218,9 +218,9 @@ sub GetRecords {
 
         my $biblioitem = $biblio->biblioitem->unblessed;
 
-        my $record = $biblio->metadata->record({ embed_items => 1 });
+        my $record = $biblio->metadata->record({ embed_items => 1, opac => 1, });
         if ($record) {
-            $biblioitem->{marcxml} = $record->as_xml_record();
+            $biblioitem->{marcxml} = $record->as_xml_record( C4::Context->preference('marcflavour') );
         }
 
         # Get most of the needed data
@@ -236,7 +236,7 @@ sub GetRecords {
         foreach my $checkout (@$checkouts) {
             delete $checkout->{'borrowernumber'};
         }
-        my @items            = $biblio->items->as_list;
+        my @items            = $biblio->items->filter_by_visible_in_opac->as_list;
 
         $biblioitem->{items}->{item} = [];
 
@@ -527,6 +527,13 @@ sub GetPatronInfo {
             # FIXME We should only retrieve what is needed in the template
             my $issue = $c->unblessed_all_relateds;
             delete $issue->{'more_subfields_xml'};
+
+            # Is the item already on hold by another user?
+            $issue->{'holds_on_item'} = Koha::Holds->search( { itemnumber => $issue->{'itemnumber'} } )->count;
+
+            # Is the record (next available item) on hold by another user?
+            $issue->{'holds_on_record'} = Koha::Holds->search( { biblionumber => $issue->{'biblionumber'} } )->count;
+
             push @checkouts, $issue
         }
         $borrower->{'loans'}->{'loan'} = \@checkouts;
@@ -635,7 +642,7 @@ sub GetServices {
     }
 
     # Renewal management
-    my @renewal = CanBookBeRenewed( $borrowernumber, $itemnumber );
+    my @renewal = CanBookBeRenewed( $patron, $item->checkout ); # TODO: Error if issue not found?
     if ( $renewal[0] ) {
         push @availablefor, 'loan renewal';
     }
@@ -679,17 +686,26 @@ sub RenewLoan {
     return { code => 'PatronNotFound' } unless $patron;
 
     # Get the item, or return an error code
-    my $itemnumber = $cgi->param('item_id');
+    my $itemnumber = $cgi->param('item_id'); # TODO: Refactor and send issue_id instead?
     my $item = Koha::Items->find($itemnumber);
-    return { code => 'RecordNotFound' } unless $item;
 
-    # Add renewal if possible
-    my @renewal = CanBookBeRenewed( $borrowernumber, $itemnumber );
-    if ( $renewal[0] ) { AddRenewal( $borrowernumber, $itemnumber, undef, undef, undef, undef, 0 ); }
+    return { code => 'RecordNotFound' } unless $item;
 
     my $issue = $item->checkout;
     return unless $issue; # FIXME should be handled
 
+    # Add renewal if possible
+    my @renewal = CanBookBeRenewed( $patron, $issue );
+    if ( $renewal[0] ) {
+        AddRenewal(
+            {
+                borrowernumber => $borrowernumber,
+                itemnumber     => $itemnumber,
+                seen           => 0
+            }
+        );
+    }
+
     # Hashref building
     my $out;
     $out->{'renewals'} = $issue->renewals_count;
@@ -952,7 +968,7 @@ sub _availability {
     my $location = $library ? $library->branchname : '';
     my $itemcallnumber = $item->itemcallnumber;
 
-    if ( $item->is_notforloan ) {
+    if ( $item->effective_not_for_loan_status ) {
         return ( $biblionumber, __('not available'), __('Not for loan'), $location, $itemcallnumber );
     } elsif ( $item->onloan ) {
         return ( $biblionumber, __('not available'), __('Checked out'), $location, $itemcallnumber );
@@ -962,6 +978,10 @@ sub _availability {
         return ( $biblionumber, __('not available'), __('Item withdrawn'), $location, $itemcallnumber );
     } elsif ( $item->damaged ) {
         return ( $biblionumber, __('not available'), __('Item damaged'), $location, $itemcallnumber );
+    } elsif ( $item->get_transfer ) {
+        return ( $biblionumber, __('not available'), __('In transit'), $location, $itemcallnumber );
+    } elsif ( $item->current_holds->next ) {
+        return ( $biblionumber, __('not available'), __('On hold'), $location, $itemcallnumber );
     } else {
         return ( $biblionumber, __('available'), undef, $location, $itemcallnumber );
     }
