@@ -27,10 +27,14 @@ use Koha::Items;
 use Koha::Libraries;
 use Koha::Patrons;
 use Koha::Checkouts;
+use Koha::TemplateUtils qw( process_tt );
+use Koha::Patron::Messages;
+use Koha::DateUtils qw(dt_from_string output_pref);
+use Date::Calc qw/Today Date_to_Days/;
 
 our $kp;    # koha patron
 
-=head1 METHODS
+=head1 Methods
 
 =cut
 
@@ -61,14 +65,31 @@ sub new {
     my $pw        = $kp->{password};
     my $flags     = C4::Members::patronflags( $kp );
     my $debarred  = $patron->is_debarred;
-    my ($day, $month, $year) = (localtime)[3,4,5];
-    my $today    = sprintf '%04d-%02d-%02d', $year+1900, $month+1, $day;
-    my $expired  = ($today gt $kp->{dateexpiry}) ? 1 : 0;
-    if ($expired) {
-        if ($kp->{opacnote} ) {
-            $kp->{opacnote} .= q{ };
+    siplog( "LOG_DEBUG", "Debarred = %s : ", ( $debarred || 'undef' ) );    # Do we need more debug info here?
+    my $expired = 0;
+    if ( $kp->{'dateexpiry'} && C4::Context->preference('NotifyBorrowerDeparture') ) {
+        my ( $today_year,   $today_month,   $today_day )   = Today();
+        my ( $warning_year, $warning_month, $warning_day ) = split /-/, $kp->{'dateexpiry'};
+        my $days_to_expiry = Date_to_Days( $warning_year, $warning_month, $warning_day ) -
+            Date_to_Days( $today_year, $today_month, $today_day );
+        my $dt         = dt_from_string( $kp->{'dateexpiry'}, 'iso' );
+        my $dateexpiry = output_pref( { dt => $dt, dateonly => 1 } );
+        if ( $days_to_expiry < 0 ) {
+
+            #borrower card has expired, warn the borrower
+            if ( $kp->{opacnote} ) {
+                $kp->{opacnote} .= q{ };
+            }
+            $kp->{opacnote} .= "Your account has expired as of $dateexpiry";
+            $expired = 1;
+        } elsif ( $days_to_expiry < C4::Context->preference('NotifyBorrowerDeparture') ) {
+
+            # borrower card soon to expire, warn the borrower
+            if ( $kp->{opacnote} ) {
+                $kp->{opacnote} .= q{ };
+            }
+            $kp->{opacnote} .= "Your card will expire on $dateexpiry";
         }
-        $kp->{opacnote} .= 'PATRON EXPIRED';
     }
     my %ilspatron;
     my $adr     = _get_address($kp);
@@ -100,6 +121,9 @@ sub new {
         }
     }
 
+    # Get currency 3 chars max
+    my $currency = substr Koha::Acquisition::Currencies->get_active->currency, 0, 3;
+
     my $circ_blocked =( C4::Context->preference('OverduesBlockCirc') ne "noblock" &&  defined $flags->{ODUES}->{itemlist} ) ? 1 : 0;
     {
     no warnings;    # any of these $kp->{fields} being concat'd could be undef
@@ -143,15 +167,18 @@ sub new {
         fine_blocked    => $fine_blocked,
         fee_limit       => $fee_limit,
         userid          => $kp->{userid},
+        currency        => $currency,
     );
     }
 
     if ( $patron->is_debarred and $patron->debarredcomment ) {
         $ilspatron{screen_msg} .= " -- " . $patron->debarredcomment;
     }
+
     if ( $circ_blocked ) {
         $ilspatron{screen_msg} .= " -- " . "Patron has overdues";
     }
+
     for (qw(EXPIRED CHARGES CREDITS GNA LOST NOTES)) {
         ($flags->{$_}) or next;
         if ($_ ne 'NOTES' and $flags->{$_}->{message}) {
@@ -164,7 +191,25 @@ sub new {
         }
     }
 
-    # FIXME: populate fine_items recall_items
+    if ( C4::Context->preference('SIP2AddOpacMessagesToScreenMessage') ) {
+        my $patron_messages = Koha::Patron::Messages->search(
+            {
+                borrowernumber => $kp->{borrowernumber},
+                message_type   => 'B',
+            }
+        );
+        my @messages_array;
+        while ( my $message = $patron_messages->next ) {
+            my $messagedt      = dt_from_string( $message->message_date, 'iso' );
+            my $formatted_date = output_pref( { dt => $messagedt, dateonly => 1 } );
+            push @messages_array, $formatted_date . ": " . $message->message;
+        }
+        if (@messages_array) {
+            $ilspatron{screen_msg} .= " Messages for you: " . join( ' / ', @messages_array );
+        }
+    }
+
+    # FIXME: populate recall_items
     $ilspatron{unavail_holds} = _get_outstanding_holds($kp->{borrowernumber});
 
     my $pending_checkouts = $patron->pending_checkouts;
@@ -255,22 +300,10 @@ sub format {
     my ( $self, $template ) = @_;
 
     if ($template) {
-        require Template;
         require Koha::Patrons;
 
-        my $tt = Template->new();
-
         my $patron = Koha::Patrons->find( $self->{borrowernumber} );
-
-        my $output;
-        eval {
-            $tt->process( \$template, { patron => $patron }, \$output );
-        };
-        if ( $@ ){
-            siplog("LOG_DEBUG", "Error processing template: $template");
-            return "";
-        }
-        return $output;
+        return process_tt( $template, { patron => $patron } );
     }
 }
 
@@ -361,6 +394,7 @@ sub x_items {
 
     my $item_list = [];
     if ($self->{$array_var}) {
+
         if ($start && $start > 1) {
             --$start;
         }
@@ -368,6 +402,7 @@ sub x_items {
             $start = 0;
         }
         if ( $end && $end < @{$self->{$array_var}} ) {
+          --$end;
         }
         else {
             $end = @{$self->{$array_var}};
@@ -401,42 +436,45 @@ sub charged_items {
     return $self->x_items('items', @_);
 }
 sub fine_items {
+
     require Koha::Database;
     require Template;
 
-    my $self = shift;
-    my $start = shift;
-    my $end = shift;
+    my $self   = shift;
+    my $start  = shift;
+    my $end    = shift;
     my $server = shift;
 
-    my @fees = Koha::Database->new()->schema()->resultset('Accountline')->search(
+    my @fees =
+      Koha::Database->new()->schema()->resultset('Accountline')->search(
         {
             borrowernumber    => $self->{borrowernumber},
             amountoutstanding => { '>' => '0' },
         }
-    );
+      );
 
     $start = $start ? $start - 1 : 0;
     $end   = $end   ? $end - 1   : scalar @fees - 1;
 
-    my $av_field_template = $server ? $server->{account}->{av_field_template} : undef;
-    $av_field_template ||= "[% accountline.description %] [% accountline.amountoutstanding | format('%.2f') %]";
-
-    my $tt = Template->new();
+    my $av_field_template =
+      $server ? $server->{account}->{av_field_template} : undef;
+    $av_field_template ||=
+"[% accountline.description %] [% accountline.amountoutstanding | format('%.2f') %]";
 
     my @return_values;
-    for ( my $i = $start; $i <= $end; $i++ ) {
+    for ( my $i = $start ; $i <= $end ; $i++ ) {
         my $fee = $fees[$i];
 
         next unless $fee;
 
-        my $output;
-        $tt->process( \$av_field_template, { accountline => $fee }, \$output );
+        my $output = process_tt( $av_field_template, { accountline => $fee } );
         push( @return_values, { barcode => $output } );
     }
 
     return \@return_values;
+
 }
+
 sub recall_items {
     my $self = shift;
     return $self->x_items('recall_items', @_);
@@ -515,21 +553,6 @@ sub charge_denied {
     return "Please contact library staff";
 }
 
-=head2 update_lastseen
-
-    $patron->update_lastseen();
-
-    Patron method to update lastseen field in borrower
-    to record that patron has been seen via sip connection
-
-=cut
-
-sub update_lastseen {
-    my $self = shift;
-    my $kohaobj = Koha::Patrons->find( $self->{borrowernumber} );
-    $kohaobj->track_login if $kohaobj; # track_login checks the pref
-}
-
 sub _get_address {
     my $patron = shift;
 
