@@ -35,6 +35,8 @@ use Koha::BackgroundJob::BatchUpdateBiblioHoldsQueue;
 use Koha::Biblio::ItemGroups;
 use Koha::Checkouts;
 use Koha::CirculationRules;
+use Koha::Courses;
+use Koha::Course::Items;
 use Koha::CoverImages;
 use Koha::Exceptions;
 use Koha::Exceptions::Checkin;
@@ -51,9 +53,11 @@ use Koha::Plugins;
 use Koha::Recalls;
 use Koha::Result::Boolean;
 use Koha::SearchEngine::Indexer;
+use Koha::Serial::Items;
 use Koha::StockRotationItem;
 use Koha::StockRotationRotas;
 use Koha::TrackedLinks;
+use Koha::Policy::Holds;
 
 use base qw(Koha::Object);
 
@@ -94,6 +98,9 @@ sub store {
         $self->itype($self->biblio->biblioitem->itemtype);
     }
 
+    # Ensure barcode is either defined or undef
+    $self->barcode(undef) if ( defined($self->barcode) && $self->barcode eq '' );
+
     $self->barcode( C4::Circulation::barcodedecode( $self->barcode ) );
 
     my $today  = dt_from_string;
@@ -130,6 +137,11 @@ sub store {
             $self->cn_sort($cn_sort);
         }
 
+        # should be quite rare when adding item
+        if ( $self->itemlost && $self->itemlost > 0 ) {    # TODO BZ34308
+            $self->_add_statistic('item_lost');
+        }
+
     } else { # ModItem
 
         $action = 'modify';
@@ -185,15 +197,22 @@ sub store {
             $self->permanent_location( $self->location );
         }
 
-        # If item was lost and has now been found,
-        # reverse any list item charges if necessary.
-        if (    exists $updated_columns{itemlost}
-            and $updated_columns{itemlost} <= 0
-            and $pre_mod_item->itemlost > 0 )
+        # TODO BZ 34308 (gt zero checks)
+        if (   exists $updated_columns{itemlost}
+            && ( !$updated_columns{itemlost} || $updated_columns{itemlost} <= 0 )
+            && ( $pre_mod_item->itemlost && $pre_mod_item->itemlost > 0 ) )
         {
+            # item found
+            # reverse any list item charges if necessary
             $self->_set_found_trigger($pre_mod_item);
+            $self->_add_statistic('item_found');
+        } elsif ( exists $updated_columns{itemlost}
+            && ( $updated_columns{itemlost} && $updated_columns{itemlost} > 0 )
+            && ( !$pre_mod_item->itemlost || $pre_mod_item->itemlost <= 0 ) )
+        {
+            # item lost
+            $self->_add_statistic('item_lost');
         }
-
     }
 
     my $result = $self->SUPER::store;
@@ -216,6 +235,22 @@ sub store {
     return $result;
 }
 
+sub _add_statistic {
+    my ( $self, $type ) = @_;
+    C4::Stats::UpdateStats(
+        {
+            borrowernumber => undef,
+            branch         => C4::Context->userenv ? C4::Context->userenv->{branch} : undef,
+            categorycode   => undef,
+            ccode          => $self->ccode,
+            itemnumber     => $self->itemnumber,
+            itemtype       => $self->effective_itemtype,
+            location       => $self->location,
+            type           => $type,
+        }
+    );
+}
+
 =head3 delete
 
 =cut
@@ -294,9 +329,8 @@ sub safe_to_delete {
 
     $error //= "not_same_branch"
       if defined C4::Context->userenv
-      && !C4::Context->IsSuperLibrarian()
-      && C4::Context->preference("IndependentBranches")
-      && ( C4::Context->userenv->{branch} ne $self->homebranch );
+      and defined C4::Context->userenv->{number}
+      and !Koha::Patrons->find( C4::Context->userenv->{number} )->can_edit_items_from( $self->homebranch );
 
     # check it doesn't have a waiting reserve
     $error //= "book_reserved"
@@ -337,7 +371,6 @@ sub move_to_deleted {
     return Koha::Database->new->schema->resultset('Deleteditem')->create($item_infos);
 }
 
-
 =head3 effective_itemtype
 
 Returns the itemtype for the item based on whether item level itemtypes are set or not.
@@ -355,8 +388,19 @@ sub effective_itemtype {
 =cut
 
 sub home_branch {
-    my ($self) = @_;
+    return shift->home_library(@_);
+}
 
+=head3 home_library
+
+my $library = $item->home_library
+
+Return the Koha::Library object representing the home library
+
+=cut
+
+sub home_library {
+    my ($self) = @_;
     my $hb_rs = $self->_result->homebranch;
 
     return Koha::Library->_new_from_dbic( $hb_rs );
@@ -367,6 +411,18 @@ sub home_branch {
 =cut
 
 sub holding_branch {
+    return shift->holding_library(@_);
+}
+
+=head3 holding_library
+
+my $library = $item->holding_library
+
+Return the Koha::Library object representing the holding library
+
+=cut
+
+sub holding_library {
     my ($self) = @_;
 
     my $hb_rs = $self->_result->holdingbranch;
@@ -417,6 +473,17 @@ sub checkout {
     return Koha::Checkout->_new_from_dbic( $checkout_rs );
 }
 
+=head3 serial_item
+
+=cut
+
+sub serial_item {
+    my ( $self ) = @_;
+    my $rs = $self->_result->serial_item;
+    return unless $rs;
+    return Koha::Serial::Item->_new_from_dbic($rs);
+}
+
 =head3 item_group
 
 my $item_group = $item->item_group;
@@ -438,6 +505,22 @@ sub item_group {
     return $item_group;
 }
 
+=head3 item_group_item
+
+    my $item_group_item = $item->item_group_item;
+
+Return the item group for this item
+
+=cut
+
+sub item_group_item {
+    my ( $self ) = @_;
+    my $rs = $self->_result->item_group_item;
+    return unless $rs;
+    return Koha::Biblio::ItemGroup::Item->_new_from_dbic($rs);
+}
+
+
 =head3 return_claims
 
   my $return_claims = $item->return_claims;
@@ -485,6 +568,156 @@ sub holds {
     return Koha::Holds->_new_from_dbic( $holds_rs );
 }
 
+=head3 bookings
+
+    my $bookings = $item->bookings();
+
+Returns the bookings attached to this item.
+
+=cut
+
+sub bookings {
+    my ( $self, $params ) = @_;
+    my $bookings_rs = $self->_result->bookings->search($params);
+    return Koha::Bookings->_new_from_dbic($bookings_rs);
+}
+
+=head3 find_booking
+
+  my $booking = $item->find_booking( { checkout_date => $now, due_date => $future_date } );
+
+Find the first booking that would conflict with the passed checkout dates for this item.
+
+FIXME: This can be simplified, it was originally intended to iterate all biblio level bookings
+to catch cases where this item may be the last available to satisfy a biblio level only booking.
+However, we dropped the biblio level functionality prior to push as bugs were found in it's
+implementation.
+
+=cut
+
+sub find_booking {
+    my ( $self, $params ) = @_;
+
+    my $checkout_date = $params->{checkout_date};
+    my $due_date      = $params->{due_date};
+    my $biblio        = $self->biblio;
+
+    my $dtf      = Koha::Database->new->schema->storage->datetime_parser;
+    my $bookings = $biblio->bookings(
+        [
+            # Proposed checkout starts during booked period
+            start_date => {
+                '-between' => [
+                    $dtf->format_datetime($checkout_date),
+                    $dtf->format_datetime($due_date)
+                ]
+            },
+
+            # Proposed checkout is due during booked period
+            end_date => {
+                '-between' => [
+                    $dtf->format_datetime($checkout_date),
+                    $dtf->format_datetime($due_date)
+                ]
+            },
+
+            # Proposed checkout would contain the booked period
+            {
+                start_date => { '<' => $dtf->format_datetime($checkout_date) },
+                end_date   => { '>' => $dtf->format_datetime($due_date) }
+            }
+        ],
+        { order_by => { '-asc' => 'start_date' } }
+    );
+
+    my $checkouts      = {};
+    my $loanable_items = {};
+    my $bookable_items = $biblio->bookable_items;
+    while ( my $item = $bookable_items->next ) {
+        $loanable_items->{ $item->itemnumber } = 1;
+        if ( my $checkout = $item->checkout ) {
+            $checkouts->{ $item->itemnumber } = dt_from_string( $checkout->date_due );
+        }
+    }
+
+    while ( my $booking = $bookings->next ) {
+
+        # Booking for this item
+        if ( defined( $booking->item_id )
+            && $booking->item_id == $self->itemnumber )
+        {
+            return $booking;
+        }
+
+        # Booking for another item
+        elsif ( defined( $booking->item_id ) ) {
+            # Due for another booking, remove from pool
+            delete $loanable_items->{ $booking->item_id };
+            next;
+
+        }
+
+        # Booking for any item
+        else {
+            # Can another item satisfy this booking?
+        }
+    }
+    return;
+}
+
+=head3 check_booking
+
+    my $bookable =
+        $item->check_booking( { start_date => $datetime, end_date => $datetime, [ booking_id => $booking_id ] } );
+
+Returns a boolean denoting whether the passed booking can be made without clashing.
+
+Optionally, you may pass a booking id to exclude from the checks; This is helpful when you are updating an existing booking.
+
+=cut
+
+sub check_booking {
+    my ( $self, $params ) = @_;
+
+    my $start_date = dt_from_string( $params->{start_date} );
+    my $end_date   = dt_from_string( $params->{end_date} );
+    my $booking_id = $params->{booking_id};
+
+    if ( my $checkout = $self->checkout ) {
+        return 0 if ( $start_date <= dt_from_string( $checkout->date_due ) );
+    }
+
+    my $dtf = Koha::Database->new->schema->storage->datetime_parser;
+
+    my $existing_bookings = $self->bookings(
+        [
+            start_date => {
+                '-between' => [
+                    $dtf->format_datetime($start_date),
+                    $dtf->format_datetime($end_date)
+                ]
+            },
+            end_date => {
+                '-between' => [
+                    $dtf->format_datetime($start_date),
+                    $dtf->format_datetime($end_date)
+                ]
+            },
+            {
+                start_date => { '<' => $dtf->format_datetime($start_date) },
+                end_date   => { '>' => $dtf->format_datetime($end_date) }
+            }
+        ]
+    );
+
+    my $bookings_count =
+        defined($booking_id)
+        ? $existing_bookings->search( { booking_id => { '!=' => $booking_id } } )->count
+        : $existing_bookings->count;
+
+    return $bookings_count ? 0 : 1;
+}
+
 =head3 request_transfer
 
   my $transfer = $item->request_transfer(
@@ -566,7 +799,29 @@ we still expect the item to end up at a final location eventually.
 sub get_transfer {
     my ($self) = @_;
 
-    return $self->get_transfers->search( {}, { rows => 1 } )->next;
+    my $transfer = $self->_result->current_branchtransfers->next;
+    return  Koha::Item::Transfer->_new_from_dbic($transfer) if $transfer;
+}
+
+=head3 transfer
+
+    my $transfer = $item->transfer;
+
+Returns the active transfer request. Returns I<undef> if no active transfer
+is found.
+
+Note: Transfers are retrieved in a Modified FIFO (First In First Out) order
+whereby the most recently sent, but not received, transfer will be returned
+if it exists, otherwise the oldest unsatisfied transfer will be returned.
+
+This allows for transfers to queue, which is the case for stock rotation and
+rotating collections where a manual transfer may need to take precedence but
+we still expect the item to end up at a final location eventually.
+
+=cut
+
+sub transfer {
+    return shift->get_transfer(@_);
 }
 
 =head3 get_transfers
@@ -589,12 +844,9 @@ we still expect the item to end up at a final location eventually.
 sub get_transfers {
     my ($self) = @_;
 
-    my $transfer_rs = $self->_result->branchtransfers;
+    my $transfer_rs = $self->_result->current_branchtransfers;
 
-    return Koha::Item::Transfers
-                ->_new_from_dbic($transfer_rs)
-                ->filter_by_current
-                ->search( {}, { order_by => [ { -desc => 'datesent' }, { -asc => 'daterequested' } ], } );
+    return Koha::Item::Transfers->_new_from_dbic($transfer_rs);
 }
 
 =head3 last_returned_by
@@ -668,7 +920,7 @@ sub hidden_in_opac {
 
     foreach my $field ( keys %{$rules} ) {
 
-        if ( any { $self->$field eq $_ } @{ $rules->{$field} } ) {
+        if ( any { defined $self->$field && $self->$field eq $_ } @{ $rules->{$field} } ) {
             $hidden_in_opac = 1;
             last;
         }
@@ -744,25 +996,30 @@ sub pickup_locations {
 
     my $patron = $params->{patron};
 
-    my $circ_control_branch =
-      C4::Reserves::GetReservesControlBranch( $self->unblessed(), $patron->unblessed );
-    my $branchitemrule =
-      C4::Circulation::GetBranchItemRule( $circ_control_branch, $self->itype );
+    my $circ_control_branch = Koha::Policy::Holds->holds_control_library( $self, $patron );
+    my $branchitemrule      = C4::Circulation::GetBranchItemRule( $circ_control_branch, $self->itype );
 
-    return Koha::Libraries->new()->empty if $branchitemrule->{holdallowed} eq 'from_local_hold_group' && !$self->home_branch->validate_hold_sibling( {branchcode => $patron->branchcode} );
-    return Koha::Libraries->new()->empty if $branchitemrule->{holdallowed} eq 'from_home_library' && $self->home_branch->branchcode ne $patron->branchcode;
+    if ( $branchitemrule->{holdallowed} eq 'from_local_hold_group'
+        && !$self->home_branch->validate_hold_sibling( { branchcode => $patron->branchcode } )
+        || $branchitemrule->{holdallowed} eq 'from_home_library'
+        && $self->home_branch->branchcode ne $patron->branchcode )
+    {
+        return Koha::Libraries->new()->empty;
+    }
 
-    my $pickup_libraries = Koha::Libraries->search();
-    if ($branchitemrule->{hold_fulfillment_policy} eq 'holdgroup') {
+    my $pickup_libraries;
+    if ( $branchitemrule->{hold_fulfillment_policy} eq 'holdgroup' ) {
         $pickup_libraries = $self->home_branch->get_hold_libraries;
-    } elsif ($branchitemrule->{hold_fulfillment_policy} eq 'patrongroup') {
-        my $plib = Koha::Libraries->find({ branchcode => $patron->branchcode});
+    } elsif ( $branchitemrule->{hold_fulfillment_policy} eq 'patrongroup' ) {
+        my $plib = Koha::Libraries->find( { branchcode => $patron->branchcode } );
         $pickup_libraries = $plib->get_hold_libraries;
-    } elsif ($branchitemrule->{hold_fulfillment_policy} eq 'homebranch') {
-        $pickup_libraries = Koha::Libraries->search({ branchcode => $self->homebranch });
-    } elsif ($branchitemrule->{hold_fulfillment_policy} eq 'holdingbranch') {
-        $pickup_libraries = Koha::Libraries->search({ branchcode => $self->holdingbranch });
-    };
+    } elsif ( $branchitemrule->{hold_fulfillment_policy} eq 'homebranch' ) {
+        $pickup_libraries = Koha::Libraries->search( { branchcode => $self->homebranch } );
+    } elsif ( $branchitemrule->{hold_fulfillment_policy} eq 'holdingbranch' ) {
+        $pickup_libraries = Koha::Libraries->search( { branchcode => $self->holdingbranch } );
+    } else {
+        $pickup_libraries = Koha::Libraries->search();
+    }
 
     return $pickup_libraries->search(
         {
@@ -774,7 +1031,8 @@ sub pickup_locations {
     ) unless C4::Context->preference('UseBranchTransferLimits');
 
     my $limittype = C4::Context->preference('BranchTransferLimitsType');
-    my ($ccode, $itype) = (undef, undef);
+    my $ccode;
+    my $itype;
     if( $limittype eq 'ccode' ){
         $ccode = $self->ccode;
     } else {
@@ -855,6 +1113,19 @@ sub current_holds {
     return Koha::Holds->_new_from_dbic($hold_rs);
 }
 
+=head3 first_hold
+
+    my $first_hold = $item->first_hold;
+
+Returns the first I<Koha::Hold> for the item.
+
+=cut
+
+sub first_hold {
+    my ( $self ) = @_;
+    return $self->current_holds->next;
+}
+
 =head3 stockrotationitem
 
   my $sritem = Koha::Item->stockrotationitem;
@@ -895,9 +1166,8 @@ This method checks the tmp_holdsqueue to see if this item has been selected for
 =cut
 
 sub has_pending_hold {
-    my ( $self ) = @_;
-    my $pending_hold = $self->_result->tmp_holdsqueues;
-    return $pending_hold->count ? 1: 0;
+    my ($self) = @_;
+    return $self->_result->tmp_holdsqueue ? 1 : 0;
 }
 
 =head3 has_pending_recall {
@@ -988,30 +1258,46 @@ Returns the branchcode to be recorded in statistics renewal of the item
 
 sub renewal_branchcode {
 
-    my ($self, $params ) = @_;
+    my ( $self, $params ) = @_;
 
     my $interface = C4::Context->interface;
     my $branchcode;
-    if ( $interface eq 'opac' ){
-        my $renewal_branchcode = C4::Context->preference('OpacRenewalBranch');
-        if( !defined $renewal_branchcode || $renewal_branchcode eq 'opacrenew' ){
+    my $renewal_branchcode;
+
+    if ( $interface eq 'opac' ) {
+        $renewal_branchcode = C4::Context->preference('OpacRenewalBranch');
+        if ( !defined $renewal_branchcode || $renewal_branchcode eq 'opacrenew' ) {
             $branchcode = 'OPACRenew';
         }
-        elsif ( $renewal_branchcode eq 'itemhomebranch' ) {
+    } elsif ( $interface eq 'api' ) {
+        $renewal_branchcode = C4::Context->preference('RESTAPIRenewalBranch');
+        if ( !defined $renewal_branchcode || $renewal_branchcode eq 'apirenew' ) {
+            $branchcode = 'APIRenew';
+        }
+    }
+
+    return $branchcode if $branchcode;
+
+    if ($renewal_branchcode) {
+        if ( $renewal_branchcode eq 'itemhomebranch' ) {
             $branchcode = $self->homebranch;
-        }
-        elsif ( $renewal_branchcode eq 'patronhomebranch' ) {
+        } elsif ( $renewal_branchcode eq 'patronhomebranch' ) {
             $branchcode = $self->checkout->patron->branchcode;
-        }
-        elsif ( $renewal_branchcode eq 'checkoutbranch' ) {
+        } elsif ( $renewal_branchcode eq 'checkoutbranch' ) {
             $branchcode = $self->checkout->branchcode;
-        }
-        else {
+        } elsif ( $renewal_branchcode eq 'apiuserbranch' ) {
+            $branchcode =
+                ( C4::Context->userenv && defined C4::Context->userenv->{branch} )
+                ? C4::Context->userenv->{branch}
+                : $params->{branch};
+        } else {
             $branchcode = "";
         }
     } else {
-        $branchcode = ( C4::Context->userenv && defined C4::Context->userenv->{branch} )
-            ? C4::Context->userenv->{branch} : $params->{branch};
+        $branchcode =
+            ( C4::Context->userenv && defined C4::Context->userenv->{branch} )
+            ? C4::Context->userenv->{branch}
+            : $params->{branch};
     }
     return $branchcode;
 }
@@ -1025,9 +1311,19 @@ Return the cover images associated with this item.
 sub cover_images {
     my ( $self ) = @_;
 
-    my $cover_image_rs = $self->_result->cover_images;
-    return unless $cover_image_rs;
-    return Koha::CoverImages->_new_from_dbic($cover_image_rs);
+    my $cover_images_rs = $self->_result->cover_images;
+    return Koha::CoverImages->_new_from_dbic($cover_images_rs);
+}
+
+=head3 cover_image_ids
+
+Return the cover image ids associated with this item.
+
+=cut
+
+sub cover_image_ids {
+    my ($self) = @_;
+    return [ $self->cover_images->get_column('imagenumber') ];
 }
 
 =head3 columns_to_str
@@ -1448,9 +1744,7 @@ sub to_api {
 
     $overrides->{effective_item_type_id} = $self->effective_itemtype;
 
-    my $itype_notforloan = $self->itemtype->notforloan;
-    $overrides->{effective_not_for_loan_status} =
-        ( defined $itype_notforloan && !$self->notforloan ) ? $itype_notforloan : $self->notforloan;
+    $overrides->{effective_not_for_loan_status} = $self->effective_not_for_loan_status;
 
     return { %$response, %$overrides };
 }
@@ -1516,7 +1810,7 @@ sub to_api_mapping {
 
     my $itemtype = $item->itemtype;
 
-    Returns Koha object for effective itemtype
+Returns Koha object for effective itemtype
 
 =cut
 
@@ -1526,6 +1820,41 @@ sub itemtype {
     return Koha::ItemTypes->find( $self->effective_itemtype );
 }
 
+=head3 item_type
+
+    my $item_type = $item->item_type;
+
+Returns the effective I<Koha::ItemType> for the item.
+
+FIXME: it should either return the 'real item type' or undef if no item type
+defined. And effective_itemtype should return... the effective itemtype. Right
+now it returns an id... This is all inconsistent. And the API should make it clear
+if the attribute is part of the resource, or a calculated value i.e. if the item
+is not linked to an item type on its own, then the API response should contain
+item_type: null! And the effective item type... be another attribute. I understand
+that this complicates filtering, but some query trickery could do it in the controller.
+
+=cut
+
+sub item_type {
+    return shift->itemtype;
+}
+
+=head3 effective_not_for_loan_status
+
+  my $nfl = $item->effective_not_for_loan_status;
+
+Returns the effective not for loan status of the item
+
+=cut
+
+sub effective_not_for_loan_status {
+    my ($self) = @_;
+    my $itemtype = $self->itemtype;
+    my $itype_notforloan = $itemtype ? $itemtype->notforloan : undef;
+    return ( defined($itype_notforloan) && !$self->notforloan ) ? $itype_notforloan : $self->notforloan;
+}
+
 =head3 orders
 
   my $orders = $item->orders();
@@ -1556,6 +1885,21 @@ sub tracked_links {
     return Koha::TrackedLinks->_new_from_dbic($tracked_links);
 }
 
+=head3 course_item
+
+  my $course_item = $item->course_item;
+
+Returns a Koha::Course::Item object
+
+=cut
+
+sub course_item {
+    my ($self) = @_;
+    my $rs = $self->_result->course_item;
+    return unless $rs;
+    return Koha::Course::Item->_new_from_dbic($rs);
+}
+
 =head3 move_to_biblio
 
   $item->move_to_biblio($to_biblio[, $params]);
@@ -1640,6 +1984,33 @@ sub bundle_items {
     return Koha::Items->_new_from_dbic($rs);
 }
 
+
+=head3 bundle_items_not_lost
+
+  my $bundle_items = $item->bundle_items_not_lost;
+
+Returns the items associated with this bundle that are not lost
+
+=cut
+
+sub bundle_items_not_lost {
+    my ($self) = @_;
+    return $self->bundle_items->search( { itemlost => 0 } );
+}
+
+=head3 bundle_items_lost
+
+  my $bundle_items = $item->bundle_items_lost;
+
+Returns the items associated with this bundle that are lost
+
+=cut
+
+sub bundle_items_lost {
+    my ($self) = @_;
+    return $self->bundle_items->search( { itemlost => { '!=' => 0 } } );
+}
+
 =head3 is_bundle
 
   my $is_bundle = $item->is_bundle;
@@ -1723,6 +2094,13 @@ sub add_to_bundle {
                     }
                 }
 
+                my $holds = $bundle_item->current_holds;
+                if ($holds->count) {
+                    unless ($options->{ignore_holds}) {
+                        Koha::Exceptions::Item::Bundle::ItemHasHolds->throw();
+                    }
+                }
+
                 $self->_result->add_to_item_bundles_hosts(
                     { item => $bundle_item->itemnumber } );
 
@@ -1820,8 +2198,9 @@ sub _after_item_action_hooks {
         'after_item_action',
         {
             action  => $action,
-            item    => $self,
-            item_id => $self->itemnumber,
+            item    => $self,                                             #FIXME To be deprecated
+            item_id => $self->itemnumber,                                 #FIXME To be deprecated
+            payload => { item => $self, item_id => $self->itemnumber },
         }
     );
 }
@@ -1890,7 +2269,7 @@ sub can_be_recalled {
 
     my $branchcode = C4::Context->userenv->{'branch'};
     if ( $patron ) {
-        $branchcode = C4::Circulation::_GetCircControlBranch( $self->unblessed, $patron->unblessed );
+        $branchcode = C4::Circulation::_GetCircControlBranch( $self, $patron );
     }
 
     # Check the circulation rule for each relevant itemtype for this item
@@ -2026,34 +2405,6 @@ sub check_recalls {
     return $recall;
 }
 
-=head3 is_notforloan
-
-    my $is_notforloan = $item->is_notforloan;
-
-Determine whether or not this item is "notforloan" based on
-the item's notforloan status or its item type
-
-=cut
-
-sub is_notforloan {
-    my ( $self ) = @_;
-    my $is_notforloan = 0;
-
-    if ( $self->notforloan ){
-        $is_notforloan = 1;
-    }
-    else {
-        my $itemtype = $self->itemtype;
-        if ($itemtype){
-            if ( $itemtype->notforloan ){
-                $is_notforloan = 1;
-            }
-        }
-    }
-
-    return $is_notforloan;
-}
-
 =head3 is_denied_renewal
 
     my $is_denied_renewal = $item->is_denied_renewal;
@@ -2084,6 +2435,21 @@ sub is_denied_renewal {
     return 0;
 }
 
+=head3 analytics_count
+
+    my $analytics_count = $item->analytics_count;
+
+Return the related analytic records count.
+
+It returns 0 if I<EasyAnalyticalRecords> is disabled.
+
+=cut
+
+sub analytics_count {
+    my ($self) = @_;
+    return C4::Items::GetAnalyticsCount($self->itemnumber);
+}
+
 =head3 strings_map
 
 Returns a map of column name to string representations including the string,
@@ -2141,6 +2507,96 @@ sub strings_map {
     return $strings;
 }
 
+
+=head3 location_update_trigger
+
+    $item->location_update_trigger( $action );
+
+Updates the item location based on I<$action>. It is done like this:
+
+=over 4
+
+=item For B<checkin>, location is updated following the I<UpdateItemLocationOnCheckin> preference.
+
+=item For B<checkout>, location is updated following the I<UpdateItemLocationOnCheckout> preference.
+
+=back
+
+FIXME: It should return I<$self>. See bug 35270.
+
+=cut
+
+sub location_update_trigger {
+    my ( $self, $action ) = @_;
+
+    my ( $update_loc_rules, $messages );
+    if ( $action eq 'checkin' ) {
+        $update_loc_rules = C4::Context->yaml_preference('UpdateItemLocationOnCheckin');
+    } else {
+        $update_loc_rules = C4::Context->yaml_preference('UpdateItemLocationOnCheckout');
+    }
+
+    if ($update_loc_rules) {
+        if ( defined $update_loc_rules->{_ALL_} ) {
+            if ( $update_loc_rules->{_ALL_} eq '_PERM_' ) {
+                $update_loc_rules->{_ALL_} = $self->permanent_location;
+            }
+            if ( $update_loc_rules->{_ALL_} eq '_BLANK_' ) {
+                $update_loc_rules->{_ALL_} = '';
+            }
+            if (
+                ( defined $self->location && $self->location ne $update_loc_rules->{_ALL_} )
+                || ( !defined $self->location
+                    && $update_loc_rules->{_ALL_} ne "" )
+                )
+            {
+                $messages->{'ItemLocationUpdated'} =
+                    { from => $self->location, to => $update_loc_rules->{_ALL_} };
+                $self->location( $update_loc_rules->{_ALL_} )->store(
+                    {
+                        log_action        => 0,
+                        skip_record_index => 1,
+                        skip_holds_queue  => 1
+                    }
+                );
+            }
+        } else {
+            foreach my $key ( keys %$update_loc_rules ) {
+                if ( $update_loc_rules->{$key} eq '_PERM_' ) {
+                    $update_loc_rules->{$key} = $self->permanent_location;
+                } elsif ( $update_loc_rules->{$key} eq '_BLANK_' ) {
+                    $update_loc_rules->{$key} = '';
+                }
+                if (
+                    (
+                           defined $self->location
+                        && $self->location eq $key
+                        && $self->location ne $update_loc_rules->{$key}
+                    )
+                    || (   $key eq '_BLANK_'
+                        && ( !defined $self->location || $self->location eq '' )
+                        && $update_loc_rules->{$key} ne '' )
+                    )
+                {
+                    $messages->{'ItemLocationUpdated'} = {
+                        from => $self->location,
+                        to   => $update_loc_rules->{$key}
+                    };
+                    $self->location( $update_loc_rules->{$key} )->store(
+                        {
+                            log_action        => 0,
+                            skip_record_index => 1,
+                            skip_holds_queue  => 1
+                        }
+                    );
+                    last;
+                }
+            }
+        }
+    }
+    return $messages;
+}
+
 =head3 _type
 
 =cut
