@@ -26,6 +26,7 @@ use C4::Reserves;
 use Koha::Items;
 use Koha::Patrons;
 use Koha::Holds;
+use Koha::Old::Holds;
 use Koha::DateUtils qw( dt_from_string );
 
 use List::MoreUtils qw( any );
@@ -44,12 +45,18 @@ Method that handles listing Koha::Hold objects
 sub list {
     my $c = shift->openapi->valid_input or return;
 
+    my $old = $c->param('old');
+    $c->req->params->remove('old');
+
     return try {
-        my $holds_set = Koha::Holds->new;
-        my $holds     = $c->objects->search( $holds_set );
+        my $holds_set =
+              $old
+            ? Koha::Old::Holds->new
+            : Koha::Holds->new;
+
+        my $holds = $c->objects->search($holds_set);
         return $c->render( status => 200, openapi => $holds );
-    }
-    catch {
+    } catch {
         $c->unhandled_exception($_);
     };
 }
@@ -64,7 +71,7 @@ sub add {
     my $c = shift->openapi->valid_input or return;
 
     return try {
-        my $body = $c->validation->param('body');
+        my $body = $c->req->json;
 
         my $biblio;
         my $item;
@@ -107,10 +114,7 @@ sub add {
             $item = Koha::Items->find($item_id);
 
             unless ($item) {
-                return $c->render(
-                    status  => 404,
-                    openapi => { error => "item_id not found." }
-                );
+                return $c->render_resource_not_found("Item");
             }
             else {
                 $biblio = $item->biblio;
@@ -214,7 +218,7 @@ sub add {
 
         return $c->render(
             status  => 201,
-            openapi => $hold->to_api
+            openapi => $c->objects->to_api($hold),
         );
     }
     catch {
@@ -245,20 +249,15 @@ sub edit {
     my $c = shift->openapi->valid_input or return;
 
     return try {
-        my $hold_id = $c->validation->param('hold_id');
-        my $hold = Koha::Holds->find( $hold_id );
+        my $hold = Koha::Holds->find( $c->param('hold_id') );
 
-        unless ($hold) {
-            return $c->render(
-                status  => 404,
-                openapi => { error => "Hold not found" }
-            );
-        }
+        return $c->render_resource_not_found("Hold")
+            unless $hold;
 
         my $overrides = $c->stash('koha.overrides');
         my $can_override = $overrides->{any} && C4::Context->preference('AllowHoldPolicyOverride');
 
-        my $body = $c->validation->output->{body};
+        my $body = $c->req->json;
 
         my $pickup_library_id = $body->{pickup_library_id};
 
@@ -281,7 +280,7 @@ sub edit {
         my $suspended_until = $body->{suspended_until} || $hold->suspend_until;
 
         my $params = {
-            reserve_id    => $hold_id,
+            reserve_id    => $hold->id,
             branchcode    => $pickup_library_id,
             rank          => $priority,
             suspend_until => $suspended_until,
@@ -293,7 +292,7 @@ sub edit {
 
         return $c->render(
             status  => 200,
-            openapi => $hold->to_api
+            openapi => $c->objects->to_api($hold),
         );
     }
     catch {
@@ -310,12 +309,10 @@ Method that handles deleting a Koha::Hold object
 sub delete {
     my $c = shift->openapi->valid_input or return;
 
-    my $hold_id = $c->validation->param('hold_id');
-    my $hold    = Koha::Holds->find($hold_id);
+    my $hold = Koha::Holds->find($c->param('hold_id'));
 
-    unless ($hold) {
-        return $c->render( status => 404, openapi => { error => "Hold not found." } );
-    }
+    return $c->render_resource_not_found("Hold")
+        unless $hold;
 
     return try {
 
@@ -333,10 +330,7 @@ sub delete {
 
         $hold->cancel;
 
-        return $c->render(
-            status  => 204,
-            openapi => q{}
-        );
+        return $c->render_resource_deleted;
     }
     catch {
         $c->unhandled_exception($_);
@@ -352,9 +346,9 @@ Method that handles suspending a hold
 sub suspend {
     my $c = shift->openapi->valid_input or return;
 
-    my $hold_id  = $c->validation->param('hold_id');
-    my $hold     = Koha::Holds->find($hold_id);
-    my $body     = $c->req->json;
+    my $hold = Koha::Holds->find( $c->param('hold_id') );
+    my $body = $c->req->json;
+
     my $end_date = ($body) ? $body->{end_date} : undef;
 
     unless ($hold) {
@@ -392,17 +386,15 @@ Method that handles resuming a hold
 sub resume {
     my $c = shift->openapi->valid_input or return;
 
-    my $hold_id = $c->validation->param('hold_id');
-    my $hold    = Koha::Holds->find($hold_id);
-    my $body    = $c->req->json;
+    my $hold = Koha::Holds->find($c->param('hold_id'));
+    my $body = $c->req->json;
 
-    unless ($hold) {
-        return $c->render( status => 404, openapi => { error => 'Hold not found.' } );
-    }
+    return $c->render_resource_not_found("Hold")
+        unless $hold;
 
     return try {
         $hold->resume;
-        return $c->render( status => 204, openapi => {} );
+        return $c->render_resource_deleted;
     }
     catch {
         $c->unhandled_exception($_);
@@ -418,21 +410,16 @@ Method that handles modifying a Koha::Hold object
 sub update_priority {
     my $c = shift->openapi->valid_input or return;
 
-    my $hold_id = $c->validation->param('hold_id');
-    my $hold = Koha::Holds->find($hold_id);
+    my $hold = Koha::Holds->find($c->param('hold_id'));
 
-    unless ($hold) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Hold not found" }
-        );
-    }
+    return $c->render_resource_not_found("Hold")
+        unless $hold;
 
     return try {
         my $priority = $c->req->json;
         C4::Reserves::_FixPriority(
             {
-                reserve_id => $hold_id,
+                reserve_id => $hold->id,
                 rank       => $priority
             }
         );
@@ -454,15 +441,10 @@ used for building the dropdown selector
 sub pickup_locations {
     my $c = shift->openapi->valid_input or return;
 
-    my $hold_id = $c->validation->param('hold_id');
-    my $hold = Koha::Holds->find( $hold_id, { prefetch => [ 'patron' ] } );
+    my $hold = Koha::Holds->find( $c->param('hold_id'), { prefetch => [ 'patron' ] } );
 
-    unless ($hold) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Hold not found" }
-        );
-    }
+    return $c->render_resource_not_found("Hold")
+        unless $hold;
 
     return try {
         my $ps_set;
@@ -520,21 +502,17 @@ Method that handles modifying the pickup location of a Koha::Hold object
 sub update_pickup_location {
     my $c = shift->openapi->valid_input or return;
 
-    my $hold_id = $c->validation->param('hold_id');
-    my $body    = $c->validation->param('body');
-    my $pickup_library_id = $body->{pickup_library_id};
+    my $hold = Koha::Holds->find($c->param('hold_id'));
 
-    my $hold = Koha::Holds->find($hold_id);
-
-    unless ($hold) {
-        return $c->render(
-            status  => 404,
-            openapi => { error => "Hold not found" }
-        );
-    }
+    return $c->render_resource_not_found("Hold")
+        unless $hold;
 
     return try {
 
+        my $body = $c->req->json;
+
+        my $pickup_library_id = $body->{pickup_library_id};
+
         my $overrides    = $c->stash('koha.overrides');
         my $can_override = $overrides->{any} && C4::Context->preference('AllowHoldPolicyOverride');
 
@@ -577,5 +555,4 @@ sub update_pickup_location {
     };
 }
 
-
 1;
