@@ -40,12 +40,10 @@ Controller function that handles retrieving a patron's account balance
 sub get {
     my $c = shift->openapi->valid_input or return;
 
-    my $patron_id = $c->validation->param('patron_id');
-    my $patron    = Koha::Patrons->find($patron_id);
+    my $patron = Koha::Patrons->find( $c->param('patron_id') );
 
-    unless ($patron) {
-        return $c->render( status => 404, openapi => { error => "Patron not found." } );
-    }
+    return $c->render_resource_not_found("Patron")
+        unless $patron;
 
     return try {
         my $account = $patron->account;
@@ -57,14 +55,14 @@ sub get {
         return $c->render(
             status  => 200,
             openapi => {
-                balance => $account->balance,
+                balance            => $account->balance,
                 outstanding_debits => {
                     total => $debits->total_outstanding,
-                    lines => $debits->to_api
+                    lines => $c->objects->to_api($debits),
                 },
                 outstanding_credits => {
                     total => $credits->total_outstanding,
-                    lines => $credits->to_api
+                    lines => $c->objects->to_api($credits),
                 }
             }
         );
@@ -74,6 +72,27 @@ sub get {
     };
 }
 
+=head3 list_credits
+
+=cut
+
+sub list_credits {
+    my $c = shift->openapi->valid_input or return;
+
+    my $patron = Koha::Patrons->find( $c->param('patron_id') );
+
+    return $c->render_resource_not_found("Patron")
+        unless $patron;
+
+    return try {
+        my $credits = $c->objects->search( $patron->account->credits );
+        return $c->render( status => 200, openapi => $credits );
+    }
+    catch {
+        $c->unhandled_exception($_);
+    };
+}
+
 =head3 add_credit
 
 Controller function that handles adding a credit to a patron's account
@@ -83,23 +102,22 @@ Controller function that handles adding a credit to a patron's account
 sub add_credit {
     my $c = shift->openapi->valid_input or return;
 
-    my $patron_id = $c->validation->param('patron_id');
-    my $patron    = Koha::Patrons->find($patron_id);
-    my $user      = $c->stash('koha.user');
+    my $patron = Koha::Patrons->find( $c->param('patron_id') );
+    my $user   = $c->stash('koha.user');
 
-
-    unless ($patron) {
-        return $c->render( status => 404, openapi => { error => "Patron not found." } );
-    }
+    return $c->render_resource_not_found("Patron")
+        unless $patron;
 
     my $account = $patron->account;
-    my $body    = $c->validation->param('body');
+    my $body    = $c->req->json;
 
     return try {
-        my $credit_type = $body->{credit_type} || 'PAYMENT';    # default to 'PAYMENT'
-        my $amount = $body->{amount};                           # mandatory, validated by openapi
+        my $credit_type =
+          $body->{credit_type} || 'PAYMENT';    # default to 'PAYMENT'
+        my $amount = $body->{amount};    # mandatory, validated by openapi
 
-        unless ( $amount > 0 ) {  # until we support newer JSON::Validator and thus minimumExclusive
+        unless ( $amount > 0 )
+        {    # until we support newer JSON::Validator and thus minimumExclusive
             Koha::Exceptions::BadParameter->throw( { parameter => 'amount' } );
         }
 
@@ -114,7 +132,8 @@ sub add_credit {
         }
 
         my $credit = $account->add_credit(
-            {   amount       => $amount,
+            {
+                amount       => $amount,
                 type         => $credit_type,
                 payment_type => $payment_type,
                 description  => $description,
@@ -127,29 +146,31 @@ sub add_credit {
         $credit->discard_changes;
 
         my $date = $body->{date};
-        $credit->date( $date )->store
-            if $date;
+        $credit->date($date)->store
+          if $date;
 
         my $debits_ids = $body->{account_lines_ids};
         my $debits;
-        $debits = Koha::Account::Lines->search({ accountlines_id => { -in => $debits_ids } })
-            if $debits_ids;
+        $debits = Koha::Account::Lines->search(
+            { accountlines_id => { -in => $debits_ids } } )
+          if $debits_ids;
 
         if ($debits) {
+
             # pay them!
-            $credit = $credit->apply({ debits => [ $debits->as_list ] });
+            $credit = $credit->apply( { debits => [ $debits->as_list ] } );
         }
 
-        if ($credit->amountoutstanding != 0) {
+        if ( $credit->amountoutstanding != 0 ) {
             my $outstanding_debits = $account->outstanding_debits;
-            $credit->apply({ debits => [ $outstanding_debits->as_list ] });
+            $credit->apply( { debits => [ $outstanding_debits->as_list ] } );
         }
 
         $credit->discard_changes;
 
         return $c->render(
             status  => 201,
-            openapi => $credit->to_api
+            openapi => $c->objects->to_api($credit),
         );
     }
     catch {
@@ -157,4 +178,89 @@ sub add_credit {
     };
 }
 
+=head3 list_debits
+
+=cut
+
+sub list_debits {
+    my $c = shift->openapi->valid_input or return;
+
+    my $patron = Koha::Patrons->find( $c->param('patron_id') );
+
+    return $c->render_resource_not_found("Patron")
+        unless $patron;
+
+    return try {
+        my $debits = $c->objects->search( $patron->account->debits );
+        return $c->render( status => 200, openapi => $debits );
+    }
+    catch {
+        $c->unhandled_exception($_);
+    };
+}
+
+=head3 add_debit
+
+=cut
+
+sub add_debit {
+    my $c = shift->openapi->valid_input or return;
+
+    my $patron = Koha::Patrons->find( $c->param('patron_id') );
+    my $user   = $c->stash('koha.user');
+
+    return $c->render_resource_not_found("Patron")
+      unless $patron;
+
+    return try {
+        my $data =
+          Koha::Account::Debit->new_from_api( $c->req->json )->unblessed;
+
+        $data->{library_id}       = delete $data->{branchcode};
+        $data->{type}             = delete $data->{debit_type_code};
+        $data->{cash_register}    = delete $data->{register_id};
+        $data->{item_id}          = delete $data->{itemnumber};
+        $data->{transaction_type} = delete $data->{payment_type};
+        $data->{interface}        = 'api'
+          ; # Should this always be API, or should we allow the API consumer to choose?
+        $data->{user_id} = delete $data->{manager_id} || $user->id;
+
+        my $debit = $patron->account->add_debit($data);
+        $debit = Koha::Account::Debit->_new_from_dbic( $debit->{_result} );
+
+        $c->res->headers->location(
+            $c->req->url->to_string . '/' . $debit->id );
+        $debit->discard_changes;
+
+        return $c->render(
+            status  => 201,
+            openapi => $c->objects->to_api($debit),
+        );
+    }
+    catch {
+        if ( blessed $_ ) {
+            if ( $_->isa('Koha::Exceptions::Account::RegisterRequired') ) {
+                return $c->render(
+                    status  => 400,
+                    openapi => { error => $_->description }
+                );
+            }
+            elsif ( $_->isa('Koha::Exceptions::Account::AmountNotPositive') ) {
+                return $c->render(
+                    status  => 400,
+                    openapi => { error => $_->description }
+                );
+            }
+            elsif ( $_->isa('Koha::Exceptions::Account::UnrecognisedType') ) {
+                return $c->render(
+                    status  => 400,
+                    openapi => { error => $_->description }
+                );
+            }
+        }
+
+        $c->unhandled_exception($_);
+    };
+}
+
 1;
