@@ -23,9 +23,9 @@
 
 use Modern::Perl;
 use CGI qw ( -utf8 );
-use YAML::XS;
 use List::MoreUtils;
 use Encode;
+use Scalar::Util qw( looks_like_number );
 
 use C4::Context;
 use C4::Auth qw( get_template_and_user );
@@ -122,7 +122,7 @@ if ($op eq ""){
 #
 # 3rd step = import the records
 #
-} elsif ( $op eq 'import_records' ) {
+} elsif ( $op eq 'cud-import_records' ) {
 #import selected lines
     $template->param('basketno' => $cgiparams->{'basketno'});
 # Budget_id is mandatory for adding an order, we just add a default, the user needs to modify this aftewards
@@ -168,9 +168,7 @@ if ($op eq ""){
         my $c_sort1 = $input->param( 'sort1_' . $import_record->import_record_id ) || $input->param('all_sort1') || '';
         my $c_sort2 = $input->param( 'sort2_' . $import_record->import_record_id ) || $input->param('all_sort2') || '';
         my $c_replacement_price = $input->param( 'replacementprice_' . $import_record->import_record_id );
-        my $c_price             = $input->param( 'price_' . $import_record->import_record_id )
-            || GetMarcPrice( $marcrecord, C4::Context->preference('marcflavour') );
-
+        my $c_price             = $input->param( 'price_' . $import_record->import_record_id );
         # Insert the biblio, or find it through matcher
         if ( $biblionumber ) { # If matched during staging we can continue
             $import_record->status('imported')->store;
@@ -213,22 +211,26 @@ if ($op eq ""){
         $import_record->import_biblio->matched_biblionumber($biblionumber)->store;
 
         # Add items from MarcItemFieldsToOrder
-        my @homebranches = $input->multi_param('homebranch_' . $import_record->import_record_id);
-        my $count = scalar @homebranches;
-        my @holdingbranches = $input->multi_param('holdingbranch_' . $import_record->import_record_id);
-        my @itypes = $input->multi_param('itype_' . $import_record->import_record_id);
-        my @nonpublic_notes = $input->multi_param('nonpublic_note_' . $import_record->import_record_id);
-        my @public_notes = $input->multi_param('public_note_' . $import_record->import_record_id);
-        my @locs = $input->multi_param('loc_' . $import_record->import_record_id);
-        my @ccodes = $input->multi_param('ccode_' . $import_record->import_record_id);
-        my @notforloans = $input->multi_param('notforloan_' . $import_record->import_record_id);
-        my @uris = $input->multi_param('uri_' . $import_record->import_record_id);
-        my @copynos = $input->multi_param('copyno_' . $import_record->import_record_id);
+        my @homebranches    = $input->multi_param( 'homebranch_' . $import_record->import_record_id );
+        my $count           = scalar @homebranches;
+        my @holdingbranches = $input->multi_param( 'holdingbranch_' . $import_record->import_record_id );
+        my @itypes          = $input->multi_param( 'itype_' . $import_record->import_record_id );
+        my @nonpublic_notes = $input->multi_param( 'nonpublic_note_' . $import_record->import_record_id );
+        my @public_notes    = $input->multi_param( 'public_note_' . $import_record->import_record_id );
+        my @locs            = $input->multi_param( 'loc_' . $import_record->import_record_id );
+        my @ccodes          = $input->multi_param( 'ccode_' . $import_record->import_record_id );
+        my @notforloans     = $input->multi_param( 'notforloan_' . $import_record->import_record_id );
+        my @uris            = $input->multi_param( 'uri_' . $import_record->import_record_id );
+        my @copynos         = $input->multi_param( 'copyno_' . $import_record->import_record_id );
         my @budget_ids =
             $input->multi_param( 'budget_code_' . $import_record->import_record_id ); # bad field name used in template!
-        my @itemprices = $input->multi_param('itemprice_' . $import_record->import_record_id);
-        my @replacementprices = $input->multi_param('replacementprice_' . $import_record->import_record_id);
-        my @itemcallnumbers = $input->multi_param('itemcallnumber_' . $import_record->import_record_id);
+        my @itemprices        = $input->multi_param( 'itemprice_' . $import_record->import_record_id );
+        my @replacementprices = $input->multi_param( 'itemreplacementprice_' . $import_record->import_record_id );
+        my @itemcallnumbers   = $input->multi_param( 'itemcallnumber_' . $import_record->import_record_id );
+        my @coded_location_qualifiers =
+            $input->multi_param( 'coded_location_qualifier_' . $import_record->import_record_id );
+        my @barcodes     = $input->multi_param( 'barcode_' . $import_record->import_record_id );
+        my @enumchrons   = $input->multi_param( 'enumchron_' . $import_record->import_record_id );
         my $itemcreation = 0;
 
         my @itemnumbers;
@@ -236,20 +238,23 @@ if ($op eq ""){
             $itemcreation = 1;
             my $item = Koha::Item->new(
                 {
-                    biblionumber        => $biblionumber,
-                    homebranch          => $homebranches[$i],
-                    holdingbranch       => $holdingbranches[$i],
-                    itemnotes_nonpublic => $nonpublic_notes[$i],
-                    itemnotes           => $public_notes[$i],
-                    location            => $locs[$i],
-                    ccode               => $ccodes[$i],
-                    itype               => $itypes[$i],
-                    notforloan          => $notforloans[$i],
-                    uri                 => $uris[$i],
-                    copynumber          => $copynos[$i],
-                    price               => $itemprices[$i],
-                    replacementprice    => $replacementprices[$i],
-                    itemcallnumber      => $itemcallnumbers[$i],
+                    biblionumber             => $biblionumber,
+                    homebranch               => $homebranches[$i],
+                    holdingbranch            => $holdingbranches[$i],
+                    itemnotes_nonpublic      => $nonpublic_notes[$i],
+                    itemnotes                => $public_notes[$i],
+                    location                 => $locs[$i],
+                    ccode                    => $ccodes[$i],
+                    itype                    => $itypes[$i],
+                    notforloan               => $notforloans[$i],
+                    uri                      => $uris[$i],
+                    copynumber               => $copynos[$i],
+                    price                    => $itemprices[$i],
+                    replacementprice         => $replacementprices[$i],
+                    itemcallnumber           => $itemcallnumbers[$i],
+                    coded_location_qualifier => $coded_location_qualifiers[$i],
+                    barcode                  => $barcodes[$i],
+                    enumchron                => $enumchrons[$i],
                 }
             )->store;
             push( @itemnumbers, $item->itemnumber );
@@ -501,7 +506,7 @@ sub import_biblios_list {
         my $marcrecord = $import_record->get_marc_record || die "couldn't translate marc information";
 
         my $infos = get_infos_syspref('MarcFieldsToOrder', $marcrecord, ['price', 'quantity', 'budget_code', 'discount', 'sort1', 'sort2','replacementprice']);
-        my $price = $infos->{price};
+        my $price = looks_like_number($infos->{price}) ? $infos->{price} : GetMarcPrice( $marcrecord, C4::Context->preference('marcflavour') );
         my $replacementprice = $infos->{replacementprice};
         my $quantity = $infos->{quantity};
         my $budget_code = $infos->{budget_code};
@@ -509,9 +514,10 @@ sub import_biblios_list {
         my $sort1 = $infos->{sort1};
         my $sort2 = $infos->{sort2};
         my $budget_id;
-        if($budget_code) {
+
+        if ($budget_code) {
             my $biblio_budget = GetBudgetByCode($budget_code);
-            if($biblio_budget) {
+            if ($biblio_budget) {
                 $budget_id = $biblio_budget->{budget_id};
             }
         }
@@ -519,56 +525,33 @@ sub import_biblios_list {
         # Items
         my @itemlist = ();
         my $all_items_quantity = 0;
-        my $alliteminfos = get_infos_syspref_on_item('MarcItemFieldsToOrder', $marcrecord, ['homebranch', 'holdingbranch', 'itype', 'nonpublic_note', 'public_note', 'loc', 'ccode', 'notforloan', 'uri', 'copyno', 'price', 'replacementprice', 'itemcallnumber', 'quantity', 'budget_code']);
+        my $alliteminfos = get_marc_item_fields_to_order($marcrecord);
         if ($alliteminfos != -1) {
             foreach my $iteminfos (@$alliteminfos) {
-                my $item_homebranch = $iteminfos->{homebranch};
-                my $item_holdingbranch = $iteminfos->{holdingbranch};
-                my $item_itype = $iteminfos->{itype};
-                my $item_nonpublic_note = $iteminfos->{nonpublic_note};
-                my $item_public_note = $iteminfos->{public_note};
-                my $item_loc = $iteminfos->{loc};
-                my $item_ccode = $iteminfos->{ccode};
-                my $item_notforloan = $iteminfos->{notforloan};
-                my $item_uri = $iteminfos->{uri};
-                my $item_copyno = $iteminfos->{copyno};
-                my $item_quantity = $iteminfos->{quantity} || 1;
-                my $item_budget_code = $iteminfos->{budget_code};
-                my $item_budget_id;
-                if ( $iteminfos->{budget_code} ) {
-                    my $item_budget = GetBudgetByCode( $iteminfos->{budget_code} );
-                    if ( $item_budget ) {
-                        $item_budget_id = $item_budget->{budget_id};
-                    }
+                # Quantity is required, default to one if not supplied
+                my $quantity = delete $iteminfos->{quantity} || 1;
+
+                # Handle incorrectly named original parameters for MarcItemFieldsToOrder
+                $iteminfos->{location}  = delete $iteminfos->{loc}   if $iteminfos->{loc};
+                # Price is handled as "itemprice" in the UI form to distinguish from MarcFieldsToOrder
+                $iteminfos->{itemprice} = delete $iteminfos->{price} if $iteminfos->{itemprice};
+
+                # Convert budge code to a budget id
+                my $item_budget_code = delete $iteminfos->{budget_code};
+                if ( $item_budget_code ) {
+                    my $item_budget = GetBudgetByCode( $item_budget_code );
+                    $iteminfos->{budget_id} = $item_budget->{budget_id} || $budget_id;
                 }
-                my $item_price = $iteminfos->{price};
-                my $item_replacement_price = $iteminfos->{replacementprice};
-                my $item_callnumber = $iteminfos->{itemcallnumber};
 
-                for (my $i = 0; $i < $item_quantity; $i++) {
+                $iteminfos->{price} ||= $price;    # Fallback to order price if no item price was defined
 
-                    my %itemrecord = (
-                        'item_id' => $item_id++,
-                        'biblio_count' => $biblio_count,
-                        'homebranch' => $item_homebranch,
-                        'holdingbranch' => $item_holdingbranch,
-                        'itype' => $item_itype,
-                        'nonpublic_note' => $item_nonpublic_note,
-                        'public_note' => $item_public_note,
-                        'loc' => $item_loc,
-                        'ccode' => $item_ccode,
-                        'notforloan' => $item_notforloan,
-                        'uri' => $item_uri,
-                        'copyno' => $item_copyno,
-                        'quantity' => $item_quantity,
-                        'budget_id' => $item_budget_id || $budget_id,
-                        'itemprice' => $item_price || $price,
-                        'replacementprice' => $item_replacement_price || $replacementprice,
-                        'itemcallnumber' => $item_callnumber,
-                    );
+                # Clone the item data for the needed quantity
+                # Add the incremented item id for each item in that quantity
+                for (my $i = 0; $i < $quantity; $i++) {
+                    my $itemrecord = { %$iteminfos };
+                    $itemrecord->{item_id} = $item_id++;
                     $all_items_quantity++;
-                    push @itemlist, \%itemrecord;
-
+                    push @itemlist, $itemrecord;
                 }
             }
 
@@ -660,15 +643,8 @@ sub add_matcher_list {
 
 sub get_infos_syspref {
     my ($syspref_name, $record, $field_list) = @_;
-    my $syspref = C4::Context->preference($syspref_name);
-    $syspref = "$syspref\n\n"; # YAML is anal on ending \n. Surplus does not hurt
-    my $yaml = eval {
-        YAML::XS::Load(Encode::encode_utf8($syspref));
-    };
-    if ( $@ ) {
-        warn "Unable to parse $syspref syspref : $@";
-        return ();
-    }
+    my $yaml = C4::Context->yaml_preference($syspref_name);
+
     my $r;
     for my $field_name ( @$field_list ) {
         next unless exists $yaml->{$field_name};
@@ -704,9 +680,10 @@ sub equal_number_of_fields {
     return $tags_count;
 }
 
-sub get_infos_syspref_on_item {
-    my ($syspref_name, $record, $field_list) = @_;
-    my $syspref = C4::Context->preference($syspref_name);
+sub get_marc_item_fields_to_order {
+    my ($record) = @_;
+
+    my $syspref = C4::Context->preference('MarcItemFieldsToOrder');
     $syspref = "$syspref\n\n"; # YAML is anal on ending \n. Surplus does not hurt
     my $yaml = eval {
         YAML::XS::Load(Encode::encode_utf8($syspref));
@@ -715,12 +692,12 @@ sub get_infos_syspref_on_item {
         warn "Unable to parse $syspref syspref : $@";
         return ();
     }
+
     my @result;
     my @tags_list;
 
     # Check tags in syspref definition
-    for my $field_name ( @$field_list ) {
-        next unless exists $yaml->{$field_name};
+    for my $field_name ( keys %$yaml ) {
         my @fields = split /\|/, $yaml->{$field_name};
         for my $field ( @fields ) {
             my ( $f, $sf ) = split /\$/, $field;
@@ -746,8 +723,7 @@ sub get_infos_syspref_on_item {
 
     for (my $i = 0; $i < $tags_count; $i++) {
         my $r;
-        for my $field_name ( @$field_list ) {
-            next unless exists $yaml->{$field_name};
+        for my $field_name ( keys %$yaml ) {
             my @fields = split /\|/, $yaml->{$field_name};
             for my $field ( @fields ) {
                 my ( $f, $sf ) = split /\$/, $field;
