@@ -41,7 +41,7 @@ use C4::Items qw( ModItemTransfer );
 use C4::Members::Messaging;
 use C4::Members;
 use C4::Output qw( output_html_with_http_headers );
-use C4::Reserves qw( ModReserve ModReserveAffect GetOtherReserves );
+use C4::Reserves qw( ModReserve ModReserveAffect CheckReserves );
 use C4::RotatingCollections;
 use Koha::AuthorisedValues;
 use Koha::BiblioFrameworks;
@@ -103,6 +103,7 @@ my $returned_counter = C4::Context->preference('numReturnedItemsToShow') || 8;
 my %returneditems;
 my %riduedate;
 my %riborrowernumber;
+my %rinot_returned;
 my @inputloop;
 foreach ( $query->param ) {
     my $counter;
@@ -120,6 +121,7 @@ foreach ( $query->param ) {
     my $barcode        = $query->param("ri-$counter");
     my $duedate        = $query->param("dd-$counter");
     my $borrowernumber = $query->param("bn-$counter");
+    my $not_returned   = $query->param("nr-$counter");
     $counter++;
 
     # decode barcode    ## Didn't we already decode them before passing them back last time??
@@ -130,23 +132,29 @@ foreach ( $query->param ) {
     $returneditems{$counter}    = $barcode;
     $riduedate{$counter}        = $duedate;
     $riborrowernumber{$counter} = $borrowernumber;
+    $rinot_returned{$counter}   = $not_returned;
 
     #######################
     $input{counter}        = $counter;
     $input{barcode}        = $barcode;
     $input{duedate}        = $duedate;
     $input{borrowernumber} = $borrowernumber;
+    $input{not_returned}   = $not_returned;
     push( @inputloop, \%input );
 }
 
+my $op          = $query->param('op') // '';
+
 ############
 # Deal with the requests....
 my $itemnumber = $query->param('itemnumber');
-if ( $query->param('reserve_id') ) {
+if ( $query->param('reserve_id') && $op eq 'cud-affect_reserve') {
     my $borrowernumber = $query->param('borrowernumber');
     my $reserve_id     = $query->param('reserve_id');
     my $diffBranchReturned = $query->param('diffBranch');
     my $cancel_reserve = $query->param('cancel_reserve');
+    my $cancel_reason = $query->param('cancel_reason');
+
     # fix up item type for display
     my $item = Koha::Items->find( $itemnumber );
     my $biblio = $item->biblio;
@@ -154,19 +162,25 @@ if ( $query->param('reserve_id') ) {
     if ( $cancel_reserve ) {
         my $hold = Koha::Holds->find( $reserve_id );
         if ( $hold ) {
-            $hold->cancel( { charge_cancel_fee => !$forgivemanualholdsexpire } );
+            $hold->cancel( { charge_cancel_fee => !$forgivemanualholdsexpire, cancellation_reason => $cancel_reason} );
         } # FIXME else?
     } else {
         my $diffBranchSend = ($userenv_branch ne $diffBranchReturned) ? $diffBranchReturned : undef;
+
         # diffBranchSend tells ModReserveAffect whether document is expected in this library or not,
         # i.e., whether to apply waiting status
         ModReserveAffect( $itemnumber, $borrowernumber, $diffBranchSend, $reserve_id, $desk_id );
-    }
-#   check if we have other reserves for this document, if we have a return send the message of transfer
-    my ( $messages, $nextreservinfo ) = GetOtherReserves($itemnumber);
 
-    my $patron = Koha::Patrons->find( $nextreservinfo );
-    if ( $messages->{'transfert'} ) {
+        if ($diffBranchSend) {
+            ModItemTransfer( $itemnumber, $userenv_branch, $diffBranchSend, 'Reserve' );
+        }
+    }
+    # check if we have other reserves for this document, if we have a result send the message of transfer
+    # FIXME do we need to do this if we didn't take the cancel_reserve branch above?
+    my ( undef, $nextreservinfo, undef ) = CheckReserves($item);
+
+    if ( $userenv_branch ne $nextreservinfo->{'branchcode'} ) {
+        my $patron = Koha::Patrons->find( $nextreservinfo->{'borrowernumber'} );
         $template->param(
             itemtitle      => $biblio->title,
             itembiblionumber => $biblio->biblionumber,
@@ -177,7 +191,7 @@ if ( $query->param('reserve_id') ) {
     }
 }
 
-if ( $query->param('recall_id') ) {
+if ( $query->param('recall_id') && $op eq 'cud-affect_recall' ) {
     my $recall = Koha::Recalls->find( scalar $query->param('recall_id') );
     my $itemnumber = $query->param('itemnumber');
     my $return_branch = $query->param('returnbranch');
@@ -212,7 +226,6 @@ if (
     undef $exemptfine;
 }
 my $dropboxmode = $query->param('dropboxmode');
-my $dotransfer  = $query->param('dotransfer');
 my $canceltransfer = $query->param('canceltransfer');
 my $transit = $query->param('transit');
 my $dest = $query->param('dest');
@@ -237,7 +250,7 @@ if ($return_date_override) {
     }
 }
 
-if ($dotransfer){
+if ( $op eq 'cud-dotransfer'){
 # An item has been returned to a branch other than the homebranch, and the librarian has chosen to initiate a transfer
     my $transferitem = $query->param('transferitem');
     my $tobranch     = $query->param('tobranch');
@@ -245,7 +258,7 @@ if ($dotransfer){
     ModItemTransfer($transferitem, $userenv_branch, $tobranch, $trigger);
 }
 
-if ($transit) {
+if ($transit && $op eq 'cud-transfer') {
     my $transfer = Koha::Item::Transfers->find($transit);
     if ( $canceltransfer ) {
         $transfer->cancel({ reason => 'Manual', force => 1});
@@ -280,7 +293,7 @@ if ($transit) {
 
 # actually return book and prepare item table.....
 my $returnbranch;
-if ($barcode) {
+if ($barcode && ( $op eq 'cud-checkin' || $op eq 'cud-affect_reserve' ) ) {
     $barcode = barcodedecode($barcode) if $barcode;
     my $item = Koha::Items->find({ barcode => $barcode });
 
@@ -298,8 +311,18 @@ if ($barcode) {
 
         # make sure return branch respects home branch circulation rules, default to homebranch
         my $hbr = Koha::CirculationRules->get_return_branch_policy($item);
-        $returnbranch = $hbr ne 'noreturn' ? $item->$hbr : $userenv_branch; # can be noreturn, homebranch or holdingbranch
+        my $validate_float =
+            Koha::Libraries->find( $item->homebranch )->validate_float_sibling( { branchcode => $userenv_branch } );
 
+        # get the proper branch to which to return the item
+        # if library isn't in same the float group, transfer item to homelibrary
+        $returnbranch =
+              $hbr eq 'noreturn'
+            ? $userenv_branch
+            : $hbr eq 'returnbylibrarygroup' ? $validate_float
+                ? $userenv_branch
+                : $item->homebranch
+            : $item->$hbr;
         my $materials = $item->materials;
         my $descriptions = Koha::AuthorisedValues->get_description_by_koha_field({frameworkcode => '', kohafield =>'items.materials', authorised_value => $materials });
         $materials = $descriptions->{lib} // $materials;
@@ -367,6 +390,7 @@ if ($barcode) {
         $returneditems{0}      = $barcode;
         $riborrowernumber{0}   = $borrower->{'borrowernumber'};
         $riduedate{0}          = $duedate;
+        $rinot_returned{0}     = 0;
         $input{borrowernumber} = $borrower->{'borrowernumber'};
         $input{duedate}        = $duedate;
         unless ( $dropboxmode ) {
@@ -401,9 +425,18 @@ if ($barcode) {
         }
 
     } elsif ( C4::Context->preference('ShowAllCheckins') and !$messages->{'BadBarcode'} and !$needs_confirm and !$bundle_confirm ) {
-        $input{duedate}   = 0;
-        $returneditems{0} = $barcode;
-        $riduedate{0}     = 0;
+        my $duedate = 0;
+        if ($issue) {
+            my $date_due_dt = dt_from_string( $issue->date_due, 'sql' );
+            $duedate               = $date_due_dt->strftime('%Y-%m-%d %H:%M');
+            $input{borrowernumber} = $issue->borrowernumber;
+            $riborrowernumber{0}   = $borrower->{'borrowernumber'};
+        }
+        $input{duedate}      = $duedate;
+        $input{not_returned} = 1;
+        $rinot_returned{0}   = 1;
+        $returneditems{0}    = $barcode;
+        $riduedate{0}        = $duedate;
         push( @inputloop, \%input );
     }
     $template->param( privacy => $borrower->{privacy} );
@@ -419,7 +452,7 @@ if ($barcode) {
     }
 
     # Mark missing bundle items as lost and report unexpected items
-    if ( $item && $item->is_bundle && $query->param('confirm_items_bundle_return') ) {
+    if ( $item && $item->is_bundle && $query->param('confirm_items_bundle_return') && !$query->param('do_not_verify_items_bundle_contents') ) {
         my $BundleLostValue = C4::Context->preference('BundleLostValue');
         my $barcodes = $query->param('verify-items-bundle-contents-barcodes');
         my @barcodes = map { s/^\s+|\s+$//gr } ( split /\n/, $barcodes );
@@ -433,7 +466,7 @@ if ($barcode) {
             $verify_item->itemlost(0);
 
             # Update last_seen
-            $verify_item->datelastseen( dt_from_string()->ymd() );
+            $verify_item->datelastseen( dt_from_string() );
 
             # Update last_borrowed if actual checkin
             $verify_item->datelastborrowed( dt_from_string()->ymd() ) if $issue;
@@ -574,15 +607,18 @@ if ( $messages->{'ResFound'} ) {
 
         my $diffBranchSend = !$branchCheck ? $reserve->{branchcode} : undef;
         ModReserveAffect( $itemnumber, $reserve->{borrowernumber}, $diffBranchSend, $reserve->{reserve_id}, $desk_id );
-        my ( $messages, $nextreservinfo ) = GetOtherReserves($reserve->{itemnumber});
+
+        if ($diffBranchSend) {
+            ModItemTransfer( $itemnumber, $item->holdingbranch, $reserve->{branchcode}, 'Reserve' );
+        }
 
         $template->param(
             hold_auto_filled => 1,
             print_slip       => C4::Context->preference('HoldsAutoFillPrintSlip'),
-            reserve_id       => $nextreservinfo->{reserve_id},
+            reserve_id       => $reserve->{reserve_id},
         );
 
-        if ( $messages->{'transfert'} ) {
+        if ($diffBranchSend) {
             $template->param(
                 itemtitle        => $biblio->title,
                 itembiblionumber => $biblio->biblionumber,
@@ -726,6 +762,9 @@ foreach my $code ( keys %$messages ) {
     }
     elsif ( $code eq 'ReturnClaims' ) {
         $template->param( ReturnClaims => $messages->{ReturnClaims} );
+    }
+      elsif ( $code eq 'ClaimAutoResolved' ) {
+          $template->param( ClaimAutoResolved => $messages->{ClaimAutoResolved} );
     } elsif ( $code eq 'RecallFound' ) {
         ;
     } elsif ( $code eq 'RecallNeedsTransfer' ) {
@@ -777,6 +816,8 @@ foreach ( sort { $a <=> $b } keys %returneditems ) {
         next unless $item; # FIXME The item has been deleted in the meantime,
                            # we could handle that better displaying a message in the template
 
+
+        $ri{not_returned} = $rinot_returned{$_};
         my $biblio = $item->biblio;
         # FIXME pass $item to the template and we are done here...
         $ri{itembiblionumber}    = $biblio->biblionumber;
@@ -798,6 +839,8 @@ foreach ( sort { $a <=> $b } keys %returneditems ) {
         $ri{homebranch}          = $item->homebranch;
         $ri{transferbranch}      = $item->get_transfer ? $item->get_transfer->tobranch : '';
         $ri{damaged}             = $item->damaged;
+        $ri{withdrawn}           = $item->withdrawn;
+        $ri{transferreason}      = $item->get_transfer ? $item->get_transfer->reason : '';
 
         $ri{location} = $item->location;
         my $shelfcode = $ri{'location'};
