@@ -4,13 +4,14 @@ INSERT INTO permissions (module_bit, code, description) VALUES
    ( 1, 'overdues_report', 'Execute overdue items report'),
    ( 1, 'force_checkout', 'Force checkout if a limitation exists'),
    ( 1, 'manage_restrictions', 'Manage restrictions for accounts'),
+   ( 1, 'manage_bookings', 'Manage item bookings'),
    ( 1, 'manage_checkout_notes', 'Mark checkout notes as seen/not seen'),
    ( 1, 'manage_curbside_pickups', 'Manage curbside pickups'),
    ( 3, 'parameters_remaining_permissions', 'Remaining system parameters permissions'),
    ( 3, 'manage_accounts', 'Manage Account Debit and Credit Types' ),
    ( 3, 'manage_circ_rules', 'Manage circulation rules'),
    ( 3, 'manage_circ_rules_from_any_libraries', 'Manage circ rules for any libraries'),
-   ( 3, 'manage_sysprefs', 'Manage global system preferences'),
+   ( 3, 'manage_sysprefs', 'Manage system preferences'),
    ( 3, 'manage_libraries', 'Manage libraries and library groups'),
    ( 3, 'manage_itemtypes', 'Manage item types'),
    ( 3, 'manage_auth_values', 'Manage authorized values'),
@@ -42,9 +43,11 @@ INSERT INTO permissions (module_bit, code, description) VALUES
    ( 3, 'manage_curbside_pickups', 'Manage curbside pickups'),
    ( 3, 'manage_search_filters', 'Manage custom search filters'),
    ( 3, 'manage_identity_providers', 'Manage identity providers'),
+   ( 3, 'manage_record_sources', 'Manage record sources'),
    ( 4, 'delete_borrowers', 'Delete patrons'),
    ( 4, 'edit_borrowers', 'Add, modify and view patron information'),
-   ( 4, 'merge_borrowers', 'Merge patrons'),
+   ( 4, 'list_borrowers', 'Search, list and view patrons'),
+   ( 4, 'send_messages_to_borrowers', 'Send messages to patrons'),
    ( 4, 'view_borrower_infos_from_any_libraries', 'View patron infos from any libraries'),
    ( 6, 'place_holds', 'Place holds for patrons'),
    ( 6, 'modify_holds_priority', 'Modify holds priority'),
@@ -58,6 +61,9 @@ INSERT INTO permissions (module_bit, code, description) VALUES
    ( 9, 'delete_all_items', 'Delete all items at once'),
    ( 9, 'manage_item_groups', 'Create, update and delete item groups, add or remove items from a item groups'),
    ( 9, 'manage_item_editor_templates', 'Update and delete item editor template owned by others'),
+   ( 9, 'edit_any_item', 'Edit any item regardless of home library'),
+   ( 9, 'edit_locked_records', 'Edit locked records'),
+   ( 9, 'set_record_sources', 'Set record source'),
    (10, 'payout', 'Perform account payout action'),
    (10, 'refund', 'Perform account refund action'),
    (10, 'discount', 'Perform account discount action'),
@@ -74,6 +80,7 @@ INSERT INTO permissions (module_bit, code, description) VALUES
    (11, 'planning_manage', 'Manage budget plannings'),
    (11, 'order_manage', 'Manage baskets and order lines'),
    (11, 'order_manage_all', 'Manage all baskets and order lines, regardless of restrictions on them'),
+   (11, 'issue_manage', 'Manage vendor issues'),
    (11, 'group_manage', 'Manage basket groups'),
    (11, 'order_receive', 'Receive orders and manage shipments'),
    (11, 'budget_add_del', 'Add and delete funds (but can''t modify funds)'),
@@ -135,9 +142,11 @@ INSERT INTO permissions (module_bit, code, description) VALUES
    (19, 'manage', 'Manage plugins ( install / uninstall )'),
    (19, 'tool', 'Use tool plugins'),
    (19, 'report', 'Use report plugins'),
+   (19, 'admin', 'Use administrative plugins'),
    (19, 'configure', 'Configure plugins'),
    (20, 'delete_public_lists', 'Delete public lists'),
    (20, 'edit_public_lists', 'Edit public lists'),
+   (20, 'edit_public_list_contents', 'Edit public list contents'),
    (21, 'edit_templates', 'Create and update club templates'),
    (21, 'edit_clubs', 'Create and update clubs'),
    (21, 'enroll', 'Enroll patrons in clubs'),
