@@ -27,7 +27,11 @@ var dataTablesDefaults = {
             "copySuccess": {
                 _: __('Copied %d rows to clipboard'),
                 1: __('Copied one row to clipboard'),
-            }
+            },
+            "print": __("Print"),
+            "copy": __("Copy"),
+            "csv": __("CSV"),
+            "excel": __("Excel")
         }
     },
     "dom": '<"dt-info"i><"top pager"<"table_entries"lp><"table_controls"fB>>tr<"bottom pager"ip>',
@@ -36,7 +40,7 @@ var dataTablesDefaults = {
         className: "dt_button_clear_filter",
         titleAttr: __('Clear filter'),
         enabled: false,
-        text: '<i class="fa fa-lg fa-remove"></i> <span class="dt-button-text">' + __('Clear filter') + '</span>',
+        text: '<i class="fa fa-lg fa-times"></i> <span class="dt-button-text">' + __('Clear filter') + '</span>',
         available: function ( dt ) {
             // The "clear filter" button is made available if this test returns true
             if( dt.settings()[0].aanFeatures.f ){ // aanFeatures.f is null if there is no search form
@@ -55,12 +59,6 @@ var dataTablesDefaults = {
         var tableId = settings.nTable.id
         var state =  settings.oLoadedState;
         state && toggledClearFilter(state.search.search, tableId);
-        // When the DataTables search function is triggered,
-        // enable or disable the "Clear filter" button based on
-        // the presence of a search string
-        $(this).on( 'search.dt', function ( e, settings ) {
-            toggledClearFilter(settings.oPreviousSearch.sSearch, tableId);
-        });
 
         if (settings.ajax) {
             let table_node = $("#" + tableId);
@@ -120,56 +118,9 @@ $.fn.dataTableExt.oApi.fnGetColumnData = function ( oSettings, iColumn, bUnique,
 // These keys must not launch filtering
 var blacklist_keys = new Array(0, 16, 17, 18, 37, 38, 39, 40);
 
-// Set a filtering delay for global search field
-jQuery.fn.dataTableExt.oApi.fnSetFilteringDelay = function ( oSettings, iDelay ) {
-    /*
-     * Inputs:      object:oSettings - dataTables settings object - automatically given
-     *              integer:iDelay - delay in milliseconds
-     * Usage:       $('#example').dataTable().fnSetFilteringDelay(250);
-     * Author:      Zygimantas Berziunas (www.zygimantas.com) and Allan Jardine
-     * License:     GPL v2 or BSD 3 point style
-     * Contact:     zygimantas.berziunas /AT\ hotmail.com
-     */
-    var
-        _that = this,
-        iDelay = (typeof iDelay == 'undefined') ? 250 : iDelay;
-
-    this.each( function ( i ) {
-        $.fn.dataTableExt.iApiIndex = i;
-        var
-            $this = this,
-            oTimerId = null,
-            sPreviousSearch = null,
-            anControl = $( 'input', _that.fnSettings().aanFeatures.f );
-
-        anControl.unbind( 'keyup.DT' ).bind( 'keyup.DT', function(event) {
-            var $$this = $this;
-            if (blacklist_keys.indexOf(event.keyCode) != -1) {
-                return this;
-            }else if ( event.keyCode == '13' ) {
-                $.fn.dataTableExt.iApiIndex = i;
-                _that.fnFilter( $(this).val() );
-            } else {
-                if (sPreviousSearch === null || sPreviousSearch != anControl.val()) {
-                    window.clearTimeout(oTimerId);
-                    sPreviousSearch = anControl.val();
-                    oTimerId = window.setTimeout(function() {
-                        $.fn.dataTableExt.iApiIndex = i;
-                        _that.fnFilter( anControl.val() );
-                    }, iDelay);
-                }
-            }
-        });
-
-        return this;
-    } );
-    return this;
-}
-
 // Add a filtering delay on general search and on all input (with a class 'filter')
 jQuery.fn.dataTableExt.oApi.fnAddFilters = function ( oSettings, sClass, iDelay ) {
     var table = this;
-    this.fnSetFilteringDelay(iDelay);
     var filterTimerId = null;
     $(table).find("input."+sClass).keyup(function(event) {
       if (blacklist_keys.indexOf(event.keyCode) != -1) {
@@ -307,8 +258,8 @@ jQuery.extend( jQuery.fn.dataTableExt.oSort, {
  * Ex: <td><span title="[% ISO_date %]">[% formatted_date %]</span></td>
  *
  * In DataTables config:
- *     "aoColumns": [
- *        { "sType": "title-string" },
+ *     "data": [
+ *        { "type": "title-string" },
  *      ]
  * http://datatables.net/plug-ins/sorting#hidden_title_string
  */
@@ -335,8 +286,8 @@ jQuery.extend( jQuery.fn.dataTableExt.oSort, {
     /* Plugin to allow text sorting to ignore articles
      *
      * In DataTables config:
-     *     "aoColumns": [
-     *        { "sType": "anti-the" },
+     *     "data": [
+     *        { "type": "anti-the" },
      *      ]
      * Based on the plugin found here:
      * http://datatables.net/plug-ins/sorting#anti_the
@@ -572,14 +523,15 @@ function _dt_default_ajax (params){
                     }
 
                     let built_value;
-                    if ( col.datatype !== undefined ) {
-                        if ( col.datatype == 'date' ) {
-                            let rfc3339 = $date_to_rfc3339(value);
-                            if ( rfc3339 != 'Invalid Date' ) {
-                                built_value = rfc3339;
-                            }
+                    if ( col.type == 'date' ) {
+                        let rfc3339 = $date_to_rfc3339(value);
+                        if ( rfc3339 != 'Invalid Date' ) {
+                            built_value = rfc3339;
                         }
-                        else if (col.datatype == 'related-object') {
+                    }
+
+                    if ( col.datatype !== undefined ) {
+                        if (col.datatype == 'related-object') {
                             let query_term = value;
 
                             if (criteria != 'exact') {
@@ -673,8 +625,12 @@ function _dt_default_ajax (params){
 
             if(query_parameters.length) {
                 query_parameters = JSON.stringify(query_parameters.length === 1?query_parameters[0]:{"-and": query_parameters});
-                dataSet.q = query_parameters;
-                delete options.query_parameters;
+                if(options.header_filter) {
+                    options.query_parameters = query_parameters;
+                } else {
+                    dataSet.q = query_parameters;
+                    delete options.query_parameters;
+                }
             } else {
                 delete options.query_parameters;
             }
@@ -731,26 +687,6 @@ function _dt_buttons(params){
         }
     }
 
-    const export_format_print = {
-        body: function (data, row, column, node) {
-            const newnode = node.cloneNode(true);
-            const no_export_nodes = newnode.querySelectorAll(".no-export");
-            no_export_nodes.forEach(child => {
-                child.parentNode.removeChild(child);
-            });
-            //Note: innerHTML is the same thing as the data variable,
-            //minus the ".no-export" nodes that we've removed
-            //Note: See dataTables.buttons.js for original function usage
-            const str = DataTable.Buttons.stripData(newnode.innerHTML, {
-                decodeEntities: false,
-                stripHtml: true,
-                stripNewlines: true,
-                trim: true,
-            });
-            return str;
-        },
-    };
-
     var export_format = {
         body: function ( data, row, column, node ) {
             var newnode = $(node);
@@ -767,7 +703,6 @@ function _dt_buttons(params){
     var export_buttons = [
         {
             extend: 'excelHtml5',
-            text: __("Excel"),
             exportOptions: {
                 columns: exportColumns,
                 format:  export_format_spreadsheet
@@ -775,7 +710,6 @@ function _dt_buttons(params){
         },
         {
             extend: 'csvHtml5',
-            text: __("CSV"),
             exportOptions: {
                 columns: exportColumns,
                 format:  export_format_spreadsheet
@@ -783,7 +717,6 @@ function _dt_buttons(params){
         },
         {
             extend: 'copyHtml5',
-            text: __("Copy"),
             exportOptions: {
                 columns: exportColumns,
                 format:  export_format
@@ -791,10 +724,9 @@ function _dt_buttons(params){
         },
         {
             extend: 'print',
-            text: __("Print"),
             exportOptions: {
                 columns: exportColumns,
-                format:  export_format_print,
+                format:  export_format
             },
         }
     ];
@@ -843,14 +775,17 @@ function _dt_buttons(params){
     );
 
     if ( table_settings && CAN_user_parameters_manage_column_config ) {
+        let href = '/cgi-bin/koha/admin/columns_settings.pl?module=%s&page=%s&table=%s'.format(table_settings.module, table_settings.page, table_settings.table);
         buttons.push(
             {
+                tag: "a",
+                attr: { href },
                 className: "dt_button_configure_table",
                 fade: 100,
                 titleAttr: __("Configure table"),
                 text: '<i class="fa fa-lg fa-wrench"></i> <span class="dt-button-text">' + __("Configure") + '</span>',
                 action: function() {
-                    window.location = '/cgi-bin/koha/admin/columns_settings.pl?module=' + table_settings['module'] + '&page=' + table_settings['page'] + '&table=' + table_settings['table'];
+                    window.location = href;
                 },
             }
         );
@@ -859,15 +794,24 @@ function _dt_buttons(params){
     return buttons;
 }
 
-function _dt_visibility(table_settings, settings){
+function _dt_visibility(table_settings, settings, node){
     var counter = 0;
     let hidden_ids = [];
     let included_ids = [];
     if ( table_settings ) {
         var columns_settings = table_settings['columns'];
         $(columns_settings).each( function() {
-            var named_id = $( 'thead th[data-colname="' + this.columnname + '"]', this ).index( 'th' );
-            var used_id = settings.bKohaColumnsUseNames ? named_id : counter;
+            let used_id = counter;
+            if ( settings.bKohaColumnsUseNames ) {
+                if (!node){
+                    console.err("settings.bKohaColumnsUseNames is set but node not passed");
+                    return;
+                }
+                let selector = '#' + node.attr('id');
+                var named_id = $( 'thead th[data-colname="' + this.columnname + '"]', selector ).index( selector + ' th' );
+                used_id = named_id;
+            }
+
             if ( used_id == -1 ) return;
 
             if ( this['is_hidden'] == "1" ) {
@@ -916,7 +860,7 @@ function _dt_add_filters(table_node, table_dt, filters_options = {}) {
                 input_type = 'select'
                 let filter_type = $(this).data('filter');
                 let existing_search = table_dt.column(i).search();
-                let select = $('<select><option value=""></option></select');
+                let select = $('<select class="dt-select-filter"><option value=""></option></select');
 
                 // FIXME eval here is bad and dangerous, how do we workaround that?
                 if ( !filters_options.hasOwnProperty(i) ) {
@@ -943,17 +887,24 @@ function _dt_add_filters(table_node, table_dt, filters_options = {}) {
                 }
             }
 
-            var search = $.fn.dataTable.util.throttle( function ( i, val ) {
-                table_dt
-                    .column( i )
-                    .search( val )
-                    .draw();
-            }, 500);
+            function delay(callback, ms) {
+                var timer = 0;
+                return function () {
+                    var context = this, args = arguments;
+                    clearTimeout(timer);
+                    timer = setTimeout(function () {
+                        callback.apply(context, args);
+                    }, ms || 0);
+                };
+            }
 
-            $( input_type, this ).on( 'keyup change', function () {
-                if ( table_dt.column(i).search() !== this.value ) {
-                    if ( input_type == "input" ) {
-                        search(i, this.value)
+            $(input_type, this).on('keyup change', (delay(function () {
+                if (table_dt.column(i).search() !== this.value) {
+                    if (input_type == "input") {
+                        table_dt
+                            .column(i)
+                            .search(this.value)
+                            .draw();
                     } else {
                         table_dt
                             .column(i)
@@ -961,7 +912,7 @@ function _dt_add_filters(table_node, table_dt, filters_options = {}) {
                             .draw();
                     }
                 }
-            } );
+            }, 500)));
         } else {
             $(this).html('');
         }
@@ -1005,17 +956,20 @@ function _dt_add_delay(table_dt, table_node, delay_ms) {
 
     /**
     * Create a new dataTables instance that uses the Koha RESTful API's as a data source
-    * @param  {Object}  options         Please see the dataTables documentation for further details
-    *                                   We extend the options set with the `criteria` key which allows
-    *                                   the developer to select the match type to be applied during searches
-    *                                   Valid keys are: `contains`, `starts_with`, `ends_with` and `exact`
-    * @param  {Object}  table_settings The arrayref as returned by TableSettings.GetTableSettings function available
-    *                                   from the columns_settings template toolkit include
-    * @param  {Boolean} add_filters     Add a filters row as the top row of the table
-    * @param  {Object}  default_filters Add a set of default search filters to apply at table initialisation
-    * @return {Object}                  The dataTables instance
+    * @param  {Object}  options                      Please see the dataTables settings documentation for further
+    *                                                details
+    * @param  {string}  [options.criteria=contains]  A koha specific extension to the dataTables settings block that
+    *                                                allows setting the 'comparison operator' used in searches
+    *                                                Supports `contains`, `starts_with`, `ends_with` and `exact` match
+    * @param  {string}  [options.columns.*.type      Data type the field is stored in so we may impose some additional
+    *                                                manipulation to search strings. Supported types are currenlty 'date'
+    * @param  {Object}  table_settings               The arrayref as returned by TableSettings.GetTableSettings function
+    *                                                available from the columns_settings template toolkit include
+    * @param  {Boolean} add_filters                  Add a filters row as the top row of the table
+    * @param  {Object}  default_filters              Add a set of default search filters to apply at table initialisation
+    * @return {Object}                               The dataTables instance
     */
-    $.fn.kohaTable = function(options, table_settings, add_filters, default_filters) {
+    $.fn.kohaTable = function(options, table_settings, add_filters, default_filters, filters_options) {
         var settings = null;
 
         if(options) {
@@ -1043,7 +997,7 @@ function _dt_add_delay(table_dt, table_node, delay_ms) {
         }
 
         let hidden_ids, included_ids;
-        [hidden_ids, included_ids] = _dt_visibility(table_settings, settings)
+        [hidden_ids, included_ids] = _dt_visibility(table_settings, settings, this)
 
         settings["buttons"] = _dt_buttons({included_ids, settings, table_settings});
 
@@ -1066,12 +1020,19 @@ function _dt_add_delay(table_dt, table_node, delay_ms) {
 
         var table_dt = table.DataTable();
         if ( add_filters ) {
-            _dt_add_filters(this, table_dt);
+            _dt_add_filters(this, table_dt, filters_options);
         }
 
-        table.DataTable().on("column-visibility.dt", function(){_dt_on_visibility(add_filters, table, table_dt);})
+        table_dt.on("column-visibility.dt", function(){_dt_on_visibility(add_filters, table, table_dt);})
             .columns( hidden_ids ).visible( false );
 
+        table_dt.on( 'search.dt', function ( e, settings ) {
+            // When the DataTables search function is triggered,
+            // enable or disable the "Clear filter" button based on
+            // the presence of a search string
+            toggledClearFilter(settings.oPreviousSearch.sSearch, settings.nTable.id);
+        });
+
         return table;
     };
 
