@@ -112,6 +112,33 @@ unless ( $biblio && $record ) {
     exit;
 }
 
+# If record should be suppressed, handle it early
+if ( C4::Context->preference('OpacSuppression') ) {
+
+    # redirect to opac-blocked info page or 404?
+    my $redirect_url;
+    if ( C4::Context->preference("OpacSuppressionRedirect") ) {
+        $redirect_url = "/cgi-bin/koha/opac-blocked.pl";
+    } else {
+        $redirect_url = "/cgi-bin/koha/errors/404.pl";
+    }
+    if ( $biblio->opac_suppressed() ) {
+
+        # if OPAC suppression by IP address
+        if ( C4::Context->preference('OpacSuppressionByIPRange') ) {
+            my $IPAddress = $ENV{'REMOTE_ADDR'};
+            my $IPRange   = C4::Context->preference('OpacSuppressionByIPRange');
+            if ( $IPAddress !~ /^$IPRange/ ) {
+                print $query->redirect($redirect_url);
+                exit;
+            }
+        } else {
+            print $query->redirect($redirect_url);
+            exit;
+        }
+    }
+}
+
 my $items = $biblio->items->search_ordered;
 if ($specific_item) {
     $items = $items->search( { itemnumber => scalar $query->param('itemnumber') } );
@@ -140,37 +167,6 @@ my $record_processor = Koha::RecordProcessor->new({
 });
 $record_processor->process($record);
 
-# redirect if opacsuppression is enabled and biblio is suppressed
-if (C4::Context->preference('OpacSuppression')) {
-    # FIXME hardcoded; the suppression flag ought to be materialized
-    # as a column on biblio or the like
-    my $opacsuppressionfield = '942';
-    my $opacsuppressionfieldvalue = $record->field($opacsuppressionfield);
-    # redirect to opac-blocked info page or 404?
-    my $opacsuppressionredirect;
-    if ( C4::Context->preference("OpacSuppressionRedirect") ) {
-        $opacsuppressionredirect = "/cgi-bin/koha/opac-blocked.pl";
-    } else {
-        $opacsuppressionredirect = "/cgi-bin/koha/errors/404.pl";
-    }
-    if ( $opacsuppressionfieldvalue &&
-         $opacsuppressionfieldvalue->subfield("n") &&
-         $opacsuppressionfieldvalue->subfield("n") == 1) {
-        # if OPAC suppression by IP address
-        if (C4::Context->preference('OpacSuppressionByIPRange')) {
-            my $IPAddress = $ENV{'REMOTE_ADDR'};
-            my $IPRange = C4::Context->preference('OpacSuppressionByIPRange');
-            if ($IPAddress !~ /^$IPRange/)  {
-                print $query->redirect($opacsuppressionredirect);
-                exit;
-            }
-        } else {
-            print $query->redirect($opacsuppressionredirect);
-            exit;
-        }
-    }
-}
-
 $template->param(
     biblio => $biblio
 );
@@ -620,6 +616,9 @@ if ( $showcomp eq 'both' || $showcomp eq 'opac' ) {
     $show_analytics = 1 if @{$biblio->get_marc_components(1)}; # count matters here, results does not
 }
 
+# Display volumes link
+my $show_volumes = @{ $biblio->get_marc_volumes(1) } ? 1 : 0;
+
 # XSLT processing of some stuff
 my $variables = {};
 my $lang = C4::Languages::getlanguage();
@@ -634,8 +633,9 @@ my @plugin_responses = Koha::Plugins->call(
 for my $plugin_variables ( @plugin_responses ) {
     $variables = { %$variables, %$plugin_variables };
 }
-$variables->{anonymous_session} = $borrowernumber ? 0 : 1;
+$variables->{anonymous_session}   = $borrowernumber ? 0 : 1;
 $variables->{show_analytics_link} = $show_analytics;
+$variables->{show_volumes_link}   = $show_volumes;
 $template->param(
     XSLTBloc => XSLTParse4Display({
         biblionumber   => $biblionumber,
@@ -658,7 +658,13 @@ if ( C4::Context->preference('OPACAcquisitionDetails' ) ) {
     };
 }
 
-my $can_item_be_reserved = 0;
+# Count the number of items that allow holds at the 'All libraries' rule level
+my $holdable_items = $biblio->items->filter_by_for_hold->count;
+
+# If we have a patron we need to check their policies for holds in the loop below
+# If we don't have a patron, then holdable items determines holdability
+my $can_holds_be_placed = $patron ? 0 : $holdable_items;
+
 my ( $itemloop_has_images, $otheritemloop_has_images );
 if ( not $viewallitems and $items->count > $max_items_to_display ) {
     $template->param(
@@ -682,7 +688,10 @@ else {
         $item_info->{holding_library_info} = $opac_info_holding->content if $opac_info_holding;
         $item_info->{home_library_info} = $opac_info_home->content if $opac_info_home;
 
-        $can_item_be_reserved = $can_item_be_reserved || $patron && IsAvailableForItemLevelRequest($item, $patron, undef);
+        # We only need to check if we haven't determined holds can be placed
+        # and if we don't have patron, we have already decided
+        $can_holds_be_placed =
+            $can_holds_be_placed || $patron && IsAvailableForItemLevelRequest( $item, $patron, undef );
 
         # get collection code description, too
         my $ccode = $item->ccode;
@@ -758,11 +767,8 @@ else {
     }
 }
 
-if( $can_item_be_reserved || CountItemsIssued($biblionumber) || $biblio->has_items_waiting_or_intransit ) {
-    $template->param( ReservableItems => 1 );
-}
-
 $template->param(
+    ReservableItems          => $can_holds_be_placed,
     itemloop_has_images      => $itemloop_has_images,
     otheritemloop_has_images => $otheritemloop_has_images,
 );
@@ -788,10 +794,8 @@ if( C4::Context->preference('ArticleRequests') ) {
     $template->param( artreqpossible => $artreqpossible );
 }
 
-my $norequests = ! $biblio->items->filter_by_for_hold->count;
     $template->param(
                      MARCNOTES               => $marcnotesarray,
-                     norequests              => $norequests,
                      itemdata_ccode          => $itemfields{ccode},
                      itemdata_materials      => $itemfields{materials},
                      itemdata_enumchron      => $itemfields{enumchron},
@@ -1194,6 +1198,8 @@ my $defaulttab =
         ? 'holdings' :
     $opac_serial_default eq 'subscriptions' && $subscriptionsnumber
         ? 'subscriptions' :
+    $opac_serial_default eq 'titlenotes' && $subscriptionsnumber
+        ? 'descriptions' :
     $opac_serial_default eq 'serialcollection' && @serialcollections > 0
         ? 'serialcollection' :
     $opac_serial_default eq 'holdings' && scalar (@itemloop) > 0
@@ -1223,22 +1229,22 @@ $template->param(
     'OpacLocationBranchToDisplay' => C4::Context->preference('OpacLocationBranchToDisplay'),
 );
 
-if ( C4::Context->preference('OPACAuthorIdentifiers') ) {
-    my @author_identifiers;
+if ( C4::Context->preference('OPACAuthorIdentifiersAndInformation') ) {
+    my @author_information;
     for my $author ( @{ $biblio->get_marc_authors } ) {
         my $authid    = $author->{authoritylink};
         my $authority = Koha::Authorities->find($authid);
         next unless $authority;
-        my $identifiers = $authority->get_identifiers;
-        next unless $identifiers && @$identifiers;
+        my $information = $authority->get_identifiers_and_information;
+        next unless $information;
         my ($name) =
           map  { $_->{value} }
           grep { $_->{code} eq 'a' ? $_ : () }
           @{ $author->{MARCAUTHOR_SUBFIELDS_LOOP} };
-        push @author_identifiers,
-          { authid => $authid, name => $name, identifiers => $identifiers };
+        push @author_information,
+          { authid => $authid, name => $name, information => $information};
     }
-    $template->param( author_identifiers => \@author_identifiers );
+    $template->param( author_information => \@author_information );
 }
 
 output_html_with_http_headers $query, $cookie, $template->output;
