@@ -32,6 +32,7 @@ use Koha::AuthorisedValues;
 use Koha::Acquisition::Currencies;
 use Koha::Libraries;
 use Koha::Patrons;
+use Koha::Suggestions;
 use Koha::Token;
 
 use URI::Escape qw( uri_escape );
@@ -94,12 +95,13 @@ my $tabcode         = $input->param('tabcode');
 my $save_confirmed  = $input->param('save_confirmed') || 0;
 my $notify          = $input->param('notify');
 my $filter_archived = $input->param('filter_archived') || 0;
+my $new_itemtype    = $input->param('suggestion_itemtype');
+my $sugg_managedby  = $input->param('suggestion_managedby');
 
 my $reasonsloop     = GetAuthorisedValues("SUGGEST");
 
-# filter informations which are not suggestion related.
-my $suggestion_ref  = { %{$input->Vars} }; # Copying, otherwise $input will be modified
-delete $suggestion_ref->{csrf_token};
+my $suggestion_ref  = { $input->Vars };
+delete $suggestion_ref->{$_} for qw(csrf_token suggestion_itemtype suggestion_managedby table_1_length);
 
 # get only the columns of Suggestion
 my $schema = Koha::Database->new()->schema;
@@ -115,8 +117,11 @@ foreach (keys %$suggestion_ref){
 }
 delete $suggestion_only->{branchcode} if $suggestion_only->{branchcode} eq '__ANY__';
 delete $suggestion_only->{budgetid}   if $suggestion_only->{budgetid}   eq '__ANY__';
-while ( my ( $k, $v ) = each %$suggestion_only ) {
-    delete $suggestion_only->{$k} if $v eq '';
+
+unless ( $op eq 'cud-save' ) {
+    while ( my ( $k, $v ) = each %$suggestion_only ) {
+        delete $suggestion_only->{$k} if $v eq '';
+    }
 }
 
 my ( $template, $borrowernumber, $cookie, $userflags ) = get_template_and_user(
@@ -136,8 +141,7 @@ my $branchfilter = $input->param('branchcode') || C4::Context->userenv->{'branch
 ##  Operations
 ##
 
-if ( $op =~ /save/i ) {
-    output_and_exit_if_error($input, $cookie, $template, { check => 'csrf_token' });
+if ( $op =~ /cud-save/ ) {
     my @messages;
     my $biblio = MarcRecordFromNewSuggestion({
             title => $suggestion_only->{title},
@@ -166,6 +170,7 @@ if ( $op =~ /save/i ) {
         delete $suggestion_ref->{suggesteddate};
         delete $suggestion_ref->{manageddate};
         Init($suggestion_ref);
+        $op = 'save';
     }
     else {
 
@@ -237,7 +242,7 @@ if ( $op =~ /save/i ) {
             }
             else {
                 ## Adding some informations related to suggestion
-                &NewSuggestion($suggestion_only);
+                Koha::Suggestion->new($suggestion_only)->store();
             }
             # empty fields, to avoid filter in "SearchSuggestion"
         }
@@ -249,14 +254,13 @@ if ( $op =~ /save/i ) {
         }
     }
 }
-elsif ($op=~/add/) {
+elsif ( $op eq 'add_form' ) {
     #Adds suggestion
     Init($suggestion_ref);
     $op ='save';
 }
-elsif ($op=~/edit/) {
+elsif ( $op eq 'edit_form' ) {
     #Edit suggestion
-    output_and_exit_if_error($input, $cookie, $template, { check => 'csrf_token' });
     $suggestion_ref=&GetSuggestion($$suggestion_ref{'suggestionid'});
     $suggestion_ref->{reasonsloop} = $reasonsloop;
     my $other_reason = 1;
@@ -270,8 +274,7 @@ elsif ($op=~/edit/) {
     Init($suggestion_ref);
     $op ='save';
 }  
-elsif ($op eq "update_status" ) {
-    output_and_exit_if_error($input, $cookie, $template, { check => 'csrf_token' });
+elsif ($op eq "cud-update_status" ) {
     my $suggestion;
     # set accepted/rejected/managed informations if applicable
     # ie= if the librarian has chosen some action on the suggestions
@@ -307,36 +310,31 @@ elsif ($op eq "update_status" ) {
         &ModSuggestion($suggestion);
     }
     redirect_with_params($input);
-}elsif ($op eq "delete" ) {
-    output_and_exit_if_error($input, $cookie, $template, { check => 'csrf_token' });
+}elsif ($op eq "cud-delete" ) {
     foreach my $delete_field (@editsuggestions) {
         &DelSuggestion( $borrowernumber, $delete_field,'intranet' );
     }
     redirect_with_params($input);
 }
-elsif ($op eq "archive" ) {
+elsif ($op eq "cud-archive" ) {
     Koha::Suggestions->find($_)->update({ archived => 1 }) for @editsuggestions;
-
     redirect_with_params($input);
 }
-elsif ($op eq "unarchive" ) {
+elsif ($op eq "cud-unarchive" ) {
     Koha::Suggestions->find($_)->update({ archived => 0 }) for @editsuggestions;
-
     redirect_with_params($input);
 }
-elsif ( $op eq 'update_itemtype' ) {
-    my $new_itemtype = $input->param('suggestion_itemtype');
+elsif ( $op eq 'cud-update_itemtype' ) {
     foreach my $suggestionid (@editsuggestions) {
         next unless $suggestionid;
         &ModSuggestion({ suggestionid => $suggestionid, itemtype => $new_itemtype });
     }
     redirect_with_params($input);
 }
-elsif ( $op eq 'update_manager' ) {
-    my $managedby = $input->param('suggestion_managedby');
+elsif ( $op eq 'cud-update_manager' ) {
     foreach my $suggestionid (@editsuggestions) {
         next unless $suggestionid;
-        &ModSuggestion({ suggestionid => $suggestionid, managedby => $managedby });
+        &ModSuggestion({ suggestionid => $suggestionid, managedby => $sugg_managedby });
     }
     redirect_with_params($input);
 }
@@ -346,8 +344,8 @@ elsif ( $op eq 'show' ) {
     $$suggestion_ref{budgetname} = $$budget{budget_name};
     Init($suggestion_ref);
 }
-if ($op=~/else/) {
-    $op='else';
+
+if ( $op eq 'else' ) {
 
     $displayby||="STATUS";
     # distinct values of display by
@@ -423,6 +421,9 @@ if ($op=~/else/) {
               if $search_params->{$f} eq '__ANY__'
               || $search_params->{$f} eq '';
         }
+        for my $bi (qw (title author isbn publishercode copyrightdate collectiontitle)) {
+            $search_params->{$bi} = { 'LIKE' => "%" . $search_params->{$bi} . "%" } if $search_params->{$bi};
+        }
 
         $search_params->{archived} = 0 if !$filter_archived;
         my @suggestions = Koha::Suggestions->search_limited($search_params)->as_list;
@@ -470,21 +471,6 @@ $template->param( returnsuggestedby => $returnsuggestedby );
 my $patron_reason_loop = GetAuthorisedValues("OPAC_SUG");
 $template->param(patron_reason_loop=>$patron_reason_loop);
 
-# Budgets for filtering
-my $budgets = GetBudgets;
-my @budgets_loop;
-foreach my $budget ( @{$budgets} ) {
-    next unless (CanUserUseBudget($borrowernumber, $budget, $userflags));
-
-    ## Please see file perltidy.ERR
-    $budget->{'selected'} = 1
-        if ($$suggestion_ref{'budgetid'}
-        && $budget->{'budget_id'} eq $$suggestion_ref{'budgetid'});
-
-    push @budgets_loop, $budget;
-}
-$template->param( budgetsloop => \@budgets_loop);
-
 # Budgets for suggestion add or edition
 my $sugg_budget_loop = [];
 my $sugg_budgets     = GetBudgetHierarchy();
@@ -534,16 +520,12 @@ foreach my $field ( qw(managedby acceptedby suggestedby budgetid) ) {
     $hashlists{ lc($field) . "_loop" } = \@codes_list;
 }
 
-my $csrf_token = Koha::Token->new->generate_csrf({
-    session_id => scalar $input->cookie('CGISESSID'),
-});
-
 $template->param(
     %hashlists,
-    borrowernumber           => ($input->param('borrowernumber') // undef),
-    SuggestionStatuses       => GetAuthorisedValues('SUGGEST_STATUS'),
-    csrf_token               => $csrf_token,
+    borrowernumber     => ( $input->param('borrowernumber') // undef ),
+    SuggestionStatuses => GetAuthorisedValues('SUGGEST_STATUS'),
 );
+
 output_html_with_http_headers $input, $cookie, $template->output;
 
 sub redirect_with_params {
