@@ -49,7 +49,6 @@ use C4::Context;
 use C4::Output qw( output_html_with_http_headers );
 use C4::Letters qw( GetMessageTransportTypes );
 use C4::Log qw( logaction );
-
 use Koha::Notice::Templates;
 use Koha::Patron::Attribute::Types;
 
@@ -100,7 +99,7 @@ $template->param(
     action             => $script_name
 );
 
-if ( $op eq 'add_validate' or $op eq 'copy_validate' ) {
+if ( $op eq 'cud-add_validate' or $op eq 'cud-copy_validate' ) {
     add_validate();
     if( $redirect eq "just_save" ){
         print $input->redirect("/cgi-bin/koha/tools/letter.pl?op=add_form&branchcode=$branchcode&module=$module&code=$code&redirect=done&section=$section&langtab=$langtab");
@@ -109,14 +108,14 @@ if ( $op eq 'add_validate' or $op eq 'copy_validate' ) {
         $op = q{}; # we return to the default screen for the next operation
     }
 }
-if ($op eq 'copy_form') {
+if ($op eq 'cud-copy_form') {
     my $oldbranchcode = $input->param('oldbranchcode') || q||;
     my $branchcode = $input->param('branchcode');
     add_form($oldbranchcode, $module, $code);
     $template->param(
         oldbranchcode => $oldbranchcode,
         branchcode => $branchcode,
-        copying => 1,
+        copy_form => 1,
         modify => 0,
     );
 }
@@ -126,7 +125,7 @@ elsif ( $op eq 'add_form' ) {
 elsif ( $op eq 'delete_confirm' ) {
     delete_confirm($branchcode, $module, $code);
 }
-elsif ( $op eq 'delete_confirmed' ) {
+elsif ( $op eq 'cud-delete_confirmed' ) {
     delete_confirmed($branchcode, $module, $code);
     $op = q{}; # next operation is to return to default screen
 }
@@ -162,33 +161,32 @@ sub add_form {
     my $templates = { map { $_ => { message_transport_type => $_ } } sort @$message_transport_types };
     my %letters = ( default => { templates => $templates } );
 
+    my $translated_languages = C4::Languages::getTranslatedLanguages(
+        'opac',
+        C4::Context->preference('template')
+    );
     if ( C4::Context->preference('TranslateNotices') ) {
-        my $translated_languages =
-          C4::Languages::getTranslatedLanguages( 'opac',
-            C4::Context->preference('template') );
         for my $language (@$translated_languages) {
-            for my $sublanguage( @{ $language->{sublanguages_loop} } ) {
+            for my $sublanguage ( @{ $language->{sublanguages_loop} } ) {
                 if ( $language->{plural} ) {
                     $letters{ $sublanguage->{rfc4646_subtag} } = {
-                        description => $sublanguage->{native_description}
-                          . ' '
-                          . $sublanguage->{region_description} . ' ('
-                          . $sublanguage->{rfc4646_subtag} . ')',
-                        templates => { %$templates },
+                              description => $sublanguage->{native_description} . ' '
+                            . $sublanguage->{region_description} . ' ('
+                            . $sublanguage->{rfc4646_subtag} . ')',
+                        templates => {%$templates},
                     };
-                }
-                else {
+                } else {
                     $letters{ $sublanguage->{rfc4646_subtag} } = {
-                        description => $sublanguage->{native_description}
-                          . ' ('
-                          . $sublanguage->{rfc4646_subtag} . ')',
-                        templates => { %$templates },
+                        description => $sublanguage->{native_description} . ' (' . $sublanguage->{rfc4646_subtag} . ')',
+                        templates   => {%$templates},
                     };
                 }
             }
         }
         $template->param( languages => $translated_languages );
     }
+    my $default_language = @{ @{$translated_languages}[0]->{sublanguages_loop} }[0]->{native_description};
+    $template->param( default_language => $default_language );
     if ($letters) {
         $template->param(
             modify     => 1,
@@ -228,6 +226,7 @@ sub add_form {
                 content    => $letter->{content} // '',
                 tt_error   => $letter->{tt_error},
             };
+            $letters{ $lang }{params} = $letter;
         }
     }
     else {
@@ -240,32 +239,29 @@ sub add_form {
 
     my $field_selection;
     push @{$field_selection}, add_fields('branches');
-    if ($module eq 'reserves') {
-        push @{$field_selection}, add_fields('borrowers', 'reserves', 'biblio', 'biblioitems', 'items');
-    }
-    elsif ( $module eq 'acquisition' ) {
-        push @{$field_selection}, add_fields('aqbooksellers', 'aqorders', 'biblio', 'items');
-    }
-    elsif ($module eq 'claimacquisition' || $module eq 'orderacquisition') {
-        push @{$field_selection}, add_fields('aqbooksellers', 'aqbasket', 'aqorders', 'biblio', 'biblioitems');
-    }
-    elsif ($module eq 'claimissues') {
-        push @{$field_selection}, add_fields('aqbooksellers', 'serial', 'subscription', 'biblio', 'biblioitems');
-    }
-    elsif ($module eq 'serial') {
-        push @{$field_selection}, add_fields('branches', 'biblio', 'biblioitems', 'borrowers', 'subscription', 'serial');
-    }
-    elsif ($module eq 'suggestions') {
-        push @{$field_selection}, add_fields('suggestions', 'borrowers', 'biblio');
-    }
-    else {
-        push @{$field_selection}, add_fields('biblio','biblioitems'),
+    if ( $module eq 'reserves' ) {
+        push @{$field_selection}, add_fields( 'borrowers', 'reserves', 'biblio', 'biblioitems', 'items' );
+    } elsif ( $module eq 'acquisition' ) {
+        push @{$field_selection}, add_fields( 'aqbooksellers', 'aqorders', 'biblio', 'items' );
+    } elsif ( $module eq 'claimacquisition' || $module eq 'orderacquisition' ) {
+        push @{$field_selection}, add_fields( 'aqbooksellers', 'aqbasket', 'aqorders', 'biblio', 'biblioitems' );
+    } elsif ( $module eq 'claimissues' ) {
+        push @{$field_selection}, add_fields( 'aqbooksellers', 'serial', 'subscription', 'biblio', 'biblioitems' );
+    } elsif ( $module eq 'patron_slip' ) {
+        push @{$field_selection}, add_fields('borrowers');
+    } elsif ( $module eq 'serial' ) {
+        push @{$field_selection},
+            add_fields( 'branches', 'biblio', 'biblioitems', 'borrowers', 'subscription', 'serial' );
+    } elsif ( $module eq 'suggestions' ) {
+        push @{$field_selection}, add_fields( 'suggestions', 'borrowers', 'biblio' );
+    } else {
+        push @{$field_selection}, add_fields( 'biblio', 'biblioitems' ),
             add_fields('items'),
-            {value => 'items.content', text => 'items.content'},
-            {value => 'items.fine',    text => 'items.fine'},
+            { value => 'items.content', text => 'items.content' },
+            { value => 'items.fine',    text => 'items.fine' },
             add_fields('borrowers');
-        if ($module eq 'circulation') {
-            push @{$field_selection}, add_fields('additional_contents', 'recalls');
+        if ( $module eq 'circulation' ) {
+            push @{$field_selection}, add_fields( 'additional_contents', 'recalls' );
 
         }
 
@@ -275,7 +271,7 @@ sub add_form {
             push @{$field_selection}, add_fields('issues');
         }
 
-        if ( $module eq 'circulation' and $code and $code =~ /^AR_/  ) {
+        if ( $module eq 'circulation' and $code and $code =~ /^AR_/ ) {
             push @{$field_selection}, add_fields('article_requests');
         }
 
@@ -315,7 +311,15 @@ sub add_validate {
     my @content       = $input->multi_param('content');
     my @lang          = $input->multi_param('lang');
     for my $mtt ( @mtt ) {
-        my $lang = shift @lang;
+        my $lang       = shift @lang;
+        my $style      = $input->param("style_$lang");
+        my $format_all = $input->param("format_all_$lang");
+        if ($format_all) {
+            my @letters = Koha::Notice::Templates->search( { lang => $lang } )->as_list;
+            foreach my $letter (@letters) {
+                $letter->set( { style => $style } )->store;
+            }
+        }
         my $is_html = $input->param("is_html_$mtt\_$lang");
         my $title   = shift @title;
         my $content = shift @content;
@@ -348,7 +352,8 @@ sub add_validate {
                     is_html    => $is_html || 0,
                     title      => $title,
                     content    => $content,
-                    lang       => $lang
+                    lang       => $lang,
+                    style      => $style
                 }
             )->store;
 
@@ -363,7 +368,8 @@ sub add_validate {
                     title                  => $title,
                     content                => $content,
                     message_transport_type => $mtt,
-                    lang                   => $lang
+                    lang                   => $lang,
+                    style                  => $style
                 }
             )->store;
             logaction( 'NOTICES', 'CREATE', $letter->id, $letter->content,
